@@ -16,7 +16,7 @@
 # Authors:
 # - Vincent Garonne <vincent.garonne@cern.ch>, 2012-2018
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2012-2021
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2012-2020
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2012-2021
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2013-2021
 # - Ralph Vigne <ralph.vigne@cern.ch>, 2013
 # - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2015-2018
@@ -32,6 +32,7 @@
 # - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
 # - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
+# - Anil Panta <anil.panta@desy.de>, 2021
 
 from __future__ import absolute_import, print_function
 
@@ -39,7 +40,14 @@ try:
     import importlib
     importlib.util.find_spec('')
 except AttributeError:
+    pass
+
+# Fallback until Python <3.9
+# Must be removed with Python 3.9
+try:
     import imp
+except AttributeError:
+    pass
 
 import base64
 import copy
@@ -211,8 +219,8 @@ def all_oidc_req_claims_present(scope, audience, required_scope, required_audien
         required_scope = ""
     if not required_audience:
         required_audience = ""
-    if (isinstance(scope, list) and isinstance(audience, list) and  # NOQA: W504
-        isinstance(required_scope, list) and isinstance(required_audience, list)):
+    if (isinstance(scope, list) and isinstance(audience, list)  # NOQA: W504
+        and isinstance(required_scope, list) and isinstance(required_audience, list)):
         scope = [str(it) for it in scope]
         audience = [str(it) for it in audience]
         required_scope = [str(it) for it in required_scope]
@@ -220,8 +228,8 @@ def all_oidc_req_claims_present(scope, audience, required_scope, required_audien
         req_scope_present = all(elem in scope for elem in required_scope)
         req_audience_present = all(elem in audience for elem in required_audience)
         return req_scope_present and req_audience_present
-    elif (isinstance(scope, string_types) and isinstance(audience, string_types) and  # NOQA: W504
-          isinstance(required_scope, string_types) and isinstance(required_audience, string_types)):
+    elif (isinstance(scope, string_types) and isinstance(audience, string_types)  # NOQA: W504
+          and isinstance(required_scope, string_types) and isinstance(required_audience, string_types)):
         scope = str(scope)
         audience = str(audience)
         required_scope = str(required_scope)
@@ -229,8 +237,8 @@ def all_oidc_req_claims_present(scope, audience, required_scope, required_audien
         req_scope_present = all(elem in scope.split(sepatator) for elem in required_scope.split(sepatator))
         req_audience_present = all(elem in audience.split(sepatator) for elem in required_audience.split(sepatator))
         return req_scope_present and req_audience_present
-    elif (isinstance(scope, list) and isinstance(audience, list) and  # NOQA: W504
-          isinstance(required_scope, string_types) and isinstance(required_audience, string_types)):
+    elif (isinstance(scope, list) and isinstance(audience, list)  # NOQA: W504
+          and isinstance(required_scope, string_types) and isinstance(required_audience, string_types)):
         scope = [str(it) for it in scope]
         audience = [str(it) for it in audience]
         required_scope = str(required_scope)
@@ -238,8 +246,8 @@ def all_oidc_req_claims_present(scope, audience, required_scope, required_audien
         req_scope_present = all(elem in scope for elem in required_scope.split(sepatator))
         req_audience_present = all(elem in audience for elem in required_audience.split(sepatator))
         return req_scope_present and req_audience_present
-    elif (isinstance(scope, string_types) and isinstance(audience, string_types) and  # NOQA: W504
-          isinstance(required_scope, list) and isinstance(required_audience, list)):
+    elif (isinstance(scope, string_types) and isinstance(audience, string_types)  # NOQA: W504
+          and isinstance(required_scope, list) and isinstance(required_audience, list)):
         scope = str(scope)
         audience = str(audience)
         required_scope = [str(it) for it in required_scope]
@@ -402,6 +410,7 @@ class APIEncoder(json.JSONEncoder):
     """ Propretary JSONEconder subclass used by the json render function.
     This is needed to address the encoding of special values.
     """
+
     def default(self, obj):  # pylint: disable=E0202
         if isinstance(obj, datetime.datetime):
             # convert any datetime to RFC 1123 format
@@ -473,7 +482,7 @@ def execute(cmd, blocking=True):
         result = process.communicate()
         (out, err) = result
         exitcode = process.returncode
-        return exitcode, out.decode(), err.decode()
+        return exitcode, out.decode(encoding='utf-8'), err.decode(encoding='utf-8')
     return process
 
 
@@ -655,9 +664,9 @@ def clean_surls(surls):
     for surl in surls:
         if surl.startswith('srm'):
             surl = re.sub(':[0-9]+/', '/', surl)
-            surl = re.sub('/srm/managerv1\?SFN=', '', surl)  # NOQA: W605
-            surl = re.sub('/srm/v2/server\?SFN=', '', surl)  # NOQA: W605
-            surl = re.sub('/srm/managerv2\?SFN=', '', surl)  # NOQA: W605
+            surl = re.sub(r'/srm/managerv1\?SFN=', '', surl)
+            surl = re.sub(r'/srm/v2/server\?SFN=', '', surl)
+            surl = re.sub(r'/srm/managerv2\?SFN=', '', surl)
         if surl.startswith('https://storage.googleapis.com'):
             surl = surl.split('?GoogleAccessId')[0]
         if '?X-Amz' in surl:
@@ -757,6 +766,8 @@ def extract_scope_belleii(did, scopes):
         return 'test', did
     if did.startswith('/belle/BG/'):
         return 'data', did
+    if did.startswith('/belle/collection'):
+        return 'collection', did
     return 'other', did
 
 
@@ -951,6 +962,9 @@ def make_valid_did(lfn_dict):
     additional `filename` key.  If this is not done, then the dictionary
     will not pass the DID JSON schema validation.
     """
+    if 'filename' not in lfn_dict:
+        return lfn_dict
+
     lfn_copy = dict(lfn_dict)
     lfn_copy['name'] = lfn_copy.get('name', lfn_copy['filename'])
     del lfn_copy['filename']
@@ -1352,7 +1366,7 @@ def setup_logger(module_name=None, logger_name=None, logger_level=None, verbose=
 
     # extracting the log level
     if not logger_level:
-        logger_level = logging.WARNING
+        logger_level = logging.INFO
         if verbose:
             logger_level = logging.DEBUG
 
