@@ -27,14 +27,17 @@
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
 # - Brandon White <bjwhite@fnal.gov>, 2019
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
+# - Matt Snyder <msnyder@bnl.gov>, 2021
+# - Radu Carpa <radu.carpa@cern.ch>, 2021
 
 import datetime
 import json
 import logging
 import time
 import traceback
-from typing import TYPE_CHECKING
+from itertools import filterfalse
+from typing import TYPE_CHECKING, Set
 
 from six import string_types
 from sqlalchemy import and_, or_, func, update
@@ -48,16 +51,19 @@ from rucio.common.utils import generate_uuid, chunks, get_parsed_throttler_mode
 from rucio.core.config import get
 from rucio.core.message import add_message
 from rucio.core.monitor import record_counter, record_timer
-from rucio.core.rse import get_rse_name, get_rse_vo, get_rse_transfer_limits
+from rucio.core.rse import get_rse_name, get_rse_vo, get_rse_transfer_limits, get_rse_attribute
 from rucio.db.sqla import models, filter_thread_work
 from rucio.db.sqla.constants import RequestState, RequestType, FTSState, ReplicaState, LockState, RequestErrMsg
 from rucio.db.sqla.session import read_session, transactional_session, stream_session
 from rucio.transfertool.fts3 import FTS3Transfertool
 
 if TYPE_CHECKING:
-    from typing import List, Tuple, Iterable, Iterator, Optional
+    from typing import List, Tuple, Iterable, Iterator, Optional, Callable, Sequence
     from sqlalchemy.orm import Session
 
+    RowIterator = Iterator[Sequence]
+    ReduceFunction = Callable[[RowIterator], RowIterator]
+
 """
 The core request.py is specifically for handling requests.
 Requests accessed by external_id (So called transfers), are covered in the core transfer.py
@@ -266,7 +272,7 @@ def queue_requests(requests, session=None, logger=logging.log):
 @read_session
 def get_next(request_type, state, limit=100, older_than=None, rse_id=None, activity=None,
              total_workers=0, worker_number=0, mode_all=False, hash_variable='id',
-             activity_shares=None, session=None):
+             activity_shares=None, transfertool=None, session=None):
     """
     Retrieve the next requests matching the request type and state.
     Workers are balanced via hashing to reduce concurrency on database.
@@ -282,6 +288,7 @@ def get_next(request_type, state, limit=100, older_than=None, rse_id=None, activ
     :param mode_all:          If set to True the function returns everything, if set to False returns list of dictionaries  {'request_id': x, 'external_host': y, 'external_id': z}.
     :param hash_variable:     The variable to use to perform the partitioning. By default it uses the request id.
     :param activity_shares:   Activity shares dictionary, with number of requests
+    :param transfertool:      The transfer tool as specified in rucio.cfg.
     :param session:           Database session to use.
     :returns:                 Request as a dictionary.
     """
@@ -304,10 +311,17 @@ def get_next(request_type, state, limit=100, older_than=None, rse_id=None, activ
 
     for share in activity_shares:
 
-        query = session.query(models.Request).with_hint(models.Request, "INDEX(REQUESTS REQUESTS_TYP_STA_UPD_IDX)", 'oracle')\
-                                             .filter(models.Request.state.in_(state))\
-                                             .filter(models.Request.request_type.in_(request_type))\
-                                             .order_by(asc(models.Request.updated_at))
+        if transfertool:
+            query = session.query(models.Request).with_hint(models.Request, "INDEX(REQUESTS REQUESTS_TYP_STA_TRA_ACT_IDX)", 'oracle')\
+                                                 .filter(models.Request.state.in_(state))\
+                                                 .filter(models.Request.request_type.in_(request_type))\
+                                                 .filter(models.Request.transfertool == transfertool)\
+                                                 .order_by(asc(models.Request.updated_at))
+        else:
+            query = session.query(models.Request).with_hint(models.Request, "INDEX(REQUESTS REQUESTS_TYP_STA_UPD_IDX)", 'oracle')\
+                                                 .filter(models.Request.state.in_(state))\
+                                                 .filter(models.Request.request_type.in_(request_type))\
+                                                 .order_by(asc(models.Request.updated_at))
 
         if isinstance(older_than, datetime.datetime):
             query = query.filter(models.Request.updated_at < older_than)
@@ -430,7 +444,7 @@ def query_request_details(request_id, transfertool='fts3', session=None, logger=
 @transactional_session
 def set_request_state(request_id, new_state, transfer_id=None, transferred_at=None, started_at=None, staging_started_at=None, staging_finished_at=None, src_rse_id=None, err_msg=None, session=None, logger=logging.log):
     """
-    Update the state of a request. Fails silently if the request_id does not exist.
+    Update the state of a request.
 
     :param request_id:           Request-ID as a 32 character hex string.
     :param new_state:            New state as string.
@@ -480,7 +494,7 @@ def set_request_state(request_id, new_state, transfer_id=None, transferred_at=No
 @transactional_session
 def set_requests_state(request_ids, new_state, session=None, logger=logging.log):
     """
-    Bulk update the state of requests. Fails silently if the request_id does not exist.
+    Bulk update the state of requests.
 
     :param request_ids:  List of (Request-ID as a 32 character hex string).
     :param new_state:    New state as string.
@@ -497,6 +511,29 @@ def set_requests_state(request_ids, new_state, session=None, logger=logging.log)
         raise RucioException(error.args)
 
 
+@transactional_session
+def set_requests_state_if_possible(request_ids, new_state, session=None, logger=logging.log):
+    """
+    Bulk update the state of requests. Skips silently if the request_id does not exist.
+
+    :param request_ids:  List of (Request-ID as a 32 character hex string).
+    :param new_state:    New state as string.
+    :param session:      Database session to use.
+    :param logger:       Optional decorated logger that can be passed from the calling daemons or servers.
+    """
+
+    record_counter('core.request.set_requests_state_if_possible')
+
+    try:
+        for request_id in request_ids:
+            try:
+                set_request_state(request_id, new_state, session=session, logger=logger)
+            except UnsupportedOperation:
+                continue
+    except IntegrityError as error:
+        raise RucioException(error.args)
+
+
 @transactional_session
 def touch_requests_by_rule(rule_id, session=None):
     """
@@ -649,7 +686,8 @@ def archive_request(request_id, session=None):
                                                                 estimated_started_at=req['estimated_started_at'],
                                                                 estimated_at=req['estimated_at'],
                                                                 transferred_at=req['transferred_at'],
-                                                                estimated_transferred_at=req['estimated_transferred_at'])
+                                                                estimated_transferred_at=req['estimated_transferred_at'],
+                                                                transfertool=req['transfertool'])
         hist_request.save(session=session)
         try:
             time_diff = req['updated_at'] - req['created_at']
@@ -1217,7 +1255,7 @@ def release_all_waiting_requests(rse_id, activity=None, account=None, direction=
         raise RucioException(error.args)
 
 
-@read_session
+@transactional_session
 def update_requests_priority(priority, filter, session=None, logger=logging.log):
     """
     Update priority of requests.
@@ -1227,12 +1265,10 @@ def update_requests_priority(priority, filter, session=None, logger=logging.log)
     :param logger:    Optional decorated logger that can be passed from the calling daemons or servers.
     """
     try:
-        query = session.query(models.Request.id, models.Request.external_id, models.Request.external_host)\
+        query = session.query(models.Request.id, models.Request.external_id, models.Request.external_host, models.Request.state, models.ReplicaLock.state)\
             .join(models.ReplicaLock, and_(models.ReplicaLock.scope == models.Request.scope,
                                            models.ReplicaLock.name == models.Request.name,
-                                           models.ReplicaLock.rse_id == models.Request.dest_rse_id))\
-            .filter(models.Request.state == RequestState.SUBMITTED,
-                    models.ReplicaLock.state == LockState.REPLICATING)
+                                           models.ReplicaLock.rse_id == models.Request.dest_rse_id))
         if 'rule_id' in filter:
             query = query.filter(models.ReplicaLock.rule_id == filter['rule_id'])
         if 'request_id' in filter:
@@ -1247,13 +1283,17 @@ def update_requests_priority(priority, filter, session=None, logger=logging.log)
         transfertool_map = {}
         for item in query.all():
             try:
-                if item[2] not in transfertool_map:
-                    transfertool_map[item[2]] = FTS3Transfertool(external_host=item[2])
-                res = transfertool_map[item[2]].update_priority(transfer_id=item[1], priority=priority)
+                session.query(models.Request) \
+                    .filter_by(id=item[0]) \
+                    .update({'priority': priority, 'updated_at': datetime.datetime.utcnow()}, synchronize_session=False)
+                logger(logging.DEBUG, "Updated request %s priority to %s in rucio." % (item[0], priority))
+                if item[3] == RequestState.SUBMITTED and item[4] == LockState.REPLICATING:
+                    if item[2] not in transfertool_map:
+                        transfertool_map[item[2]] = FTS3Transfertool(external_host=item[2])
+                    res = transfertool_map[item[2]].update_priority(transfer_id=item[1], priority=priority)
+                    logger(logging.DEBUG, "Updated request %s priority in transfertool to %s: %s" % (item[0], priority, res['http_message']))
             except Exception:
                 logger(logging.DEBUG, "Failed to boost request %s priority: %s" % (item[0], traceback.format_exc()))
-            else:
-                logger(logging.DEBUG, "Update request %s priority to %s: %s" % (item[0], priority, res['http_message']))
     except IntegrityError as error:
         raise RucioException(error.args)
 
@@ -1510,21 +1550,31 @@ dest_rse_id_col = 10
 source_rse_id_col = 12
 distance_col = 20
 
+extra_transfertool_col = 21
+
 
 @transactional_session
-def preparer_update_requests(source_iter: "Iterable[Tuple]", session: "Optional[Session]" = None) -> int:
+def preparer_update_requests(source_iter: "Iterable[Sequence]", session: "Optional[Session]" = None) -> int:
     count = 0
     for req_source in source_iter:
-        new_state = __throttler_request_state(
-            activity=req_source[activity_col],
-            source_rse_id=req_source[source_rse_id_col],
-            dest_rse_id=req_source[dest_rse_id_col],
-            session=session,
-        )
-        session.query(models.Request).filter_by(id=req_source[request_id_col]).update({
-            models.Request.state: new_state,
-            models.Request.source_rse_id: req_source[source_rse_id_col],
-        }, synchronize_session=False)
+        update_dict = dict()
+        if len(req_source) == 2:
+            # special case where the first entry is the request id and the second is the new state
+            # (see handling of RequestState.NO_SOURCES in reduce_requests)
+            update_dict[models.Request.state] = req_source[1]
+        else:
+            update_dict[models.Request.state] = __throttler_request_state(
+                activity=req_source[activity_col],
+                source_rse_id=req_source[source_rse_id_col],
+                dest_rse_id=req_source[dest_rse_id_col],
+                session=session,
+            )
+            update_dict[models.Request.source_rse_id] = req_source[source_rse_id_col]
+
+        if len(req_source) > extra_transfertool_col:
+            update_dict[models.Request.transfertool] = req_source[extra_transfertool_col]
+
+        session.query(models.Request).filter_by(id=req_source[request_id_col]).update(update_dict, synchronize_session=False)
         count += 1
     return count
 
@@ -1564,18 +1614,97 @@ def __throttler_request_state(activity, source_rse_id, dest_rse_id, session: "Op
     return RequestState.WAITING if limit_found else RequestState.QUEUED
 
 
-def minimum_distance_requests(req_sources: "List[Tuple]") -> "Iterator[Tuple]":
-    # sort by Request.id, should be pretty quick since the database should return it this way (tim sort)
+def reduce_requests(req_sources: "List[Tuple]", sort_reduce_funcs: "List[ReduceFunction]", logger: "Callable") -> "RowIterator":
+    """
+    Reduces the passed request sources tuples by using the sort-reduce functions,
+    returning the first of the remaining items or a simple
+    (request_id, RequestState.NO_SOURCES) tuple, if no sources were found.
+    """
+    assert len(req_sources) != 0, 'parameter request sources must be non-empty'
+
+    # sort by Request.id for partitioning later
     req_sources.sort(key=lambda t: t[request_id_col])
 
-    # req_sources must be non-empty and sorted by ids at this point, see above
+    def pick_result(items: "List[Sequence]") -> "Optional[Sequence]":
+        result = items
+        debug_log = []
+        for sort_reduce in sort_reduce_funcs:
+            newresult = list(sort_reduce(result))
+            debug_log.append('filter %s removed %s' % (sort_reduce.__name__, list(filterfalse(newresult.__contains__, result))))
+            result = newresult
+
+        if len(result) == 0:
+            logger(logging.WARNING, 'all available sources were filtered for requests with id %s', items[0][request_id_col])
+            logger(logging.DEBUG, 'the following filters ran:\n' + '\n'.join(debug_log))
+        else:
+            return result[0]
+
+    def result_or_no_sources(result: "Optional[Sequence]") -> "Sequence":
+        if result is None:
+            return (cur_request_id, RequestState.NO_SOURCES)
+        else:
+            return result
+
     cur_request_id = req_sources[0][request_id_col]
-    shortest_item = req_sources[0]
+
+    # partition the req_sources by request_id and yield the best result from each group
+    current_items = [req_sources[0]]
     for idx in range(1, len(req_sources)):
         if cur_request_id != req_sources[idx][request_id_col]:
-            yield shortest_item
+            yield result_or_no_sources(pick_result(current_items))
             cur_request_id = req_sources[idx][request_id_col]
-            shortest_item = req_sources[idx]
-        elif req_sources[idx][distance_col] < shortest_item[distance_col]:
-            shortest_item = req_sources[idx]
-    yield shortest_item
+            current_items.clear()
+        current_items.append(req_sources[idx])
+    yield result_or_no_sources(pick_result(current_items))
+
+
+def get_supported_transfertools(rse_id: str, session=None) -> "Set[str]":
+    transfertool_attr = get_rse_attribute('transfertool', rse_id=rse_id, session=session)
+    if transfertool_attr:
+        result = set()
+        for attr in transfertool_attr:
+            if attr:
+                assert type(attr) == str
+                # split attribute values by comma
+                for transfertool in filter(bool, map(str.strip, attr.split(sep=','))):
+                    result.add(transfertool)
+        if result:
+            return result
+    return {'fts3', 'globus'}
+
+
+def get_transfertool_filter(
+        get_transfertools: "Callable[[str], Set[str]]" = get_supported_transfertools
+) -> "ReduceFunction":
+    def filter_requests_for_transfertools(items: "RowIterator") -> "RowIterator":
+        first = True
+        first_request_id, first_dest_rse_id, dest_rse_transfertools = None, None, None
+        for t in items:
+            if first:
+                first = False
+                first_request_id = t[request_id_col]
+                first_dest_rse_id = t[dest_rse_id_col]
+                dest_rse_transfertools = get_transfertools(first_dest_rse_id)
+            else:
+                # same request id, same request destination rse in items per call
+                assert first_request_id == t[request_id_col]
+                assert first_dest_rse_id == t[dest_rse_id_col]
+
+            src_rse_transfertools = get_transfertools(t[source_rse_id_col])
+            common_transfertools = dest_rse_transfertools.intersection(src_rse_transfertools)
+            if common_transfertools:
+                if 'fts3' in common_transfertools and 'globus' in common_transfertools:
+                    transfertool = 'fts3'
+                else:
+                    transfertool = common_transfertools.pop()
+                yield (*t, transfertool)
+
+    return filter_requests_for_transfertools
+
+
+def sort_requests_minimum_distance(items: "RowIterator") -> "RowIterator":
+    yield from sorted(items, key=lambda t: t[distance_col])
+
+
+def rse_lookup_filter(items: "RowIterator") -> "RowIterator":
+    yield from filter(lambda row: row[source_rse_id_col] is not None and row[dest_rse_id_col] is not None, items)
