@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2015-2020 CERN
+# Copyright 2015-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -25,7 +25,9 @@
 # - Brandon White <bjwhite@fnal.gov>, 2019
 # - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2020-2021
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
+# - Matt Snyder <msnyder@bnl.gov>, 2021
+# - Radu Carpa <radu.carpa@cern.ch>, 2021
 
 """
 Conveyor finisher is a daemon to update replicas and rules based on requests.
@@ -68,7 +70,7 @@ graceful_stop = threading.Event()
 region = make_region().configure('dogpile.cache.memory', expiration_time=3600)
 
 
-def finisher(once=False, sleep_time=60, activities=None, bulk=100, db_bulk=1000):
+def finisher(once=False, sleep_time=60, activities=None, bulk=100, db_bulk=1000, partition_wait_time=10):
     """
     Main loop to update the replicas and rules based on finished requests.
     """
@@ -102,7 +104,8 @@ def finisher(once=False, sleep_time=60, activities=None, bulk=100, db_bulk=1000)
     logger = formatted_logger(logging.log, prefix + '%s')
     logger(logging.INFO, 'Finisher starting - db_bulk(%i) bulk (%i)', db_bulk, bulk)
 
-    graceful_stop.wait(10)
+    if partition_wait_time:
+        graceful_stop.wait(partition_wait_time)
     while not graceful_stop.is_set():
 
         start_time = time.time()
@@ -153,7 +156,7 @@ def finisher(once=False, sleep_time=60, activities=None, bulk=100, db_bulk=1000)
             logger(logging.CRITICAL, 'Exception', exc_info=True)
 
         if once:
-            return
+            break
 
         end_time = time.time()
         time_diff = end_time - start_time
@@ -203,7 +206,7 @@ def run(once=False, total_threads=1, sleep_time=60, activities=None, bulk=100, d
 
         # Interruptible joins require a timeout.
         while threads:
-            threads = [thread.join(timeout=3.14) for thread in threads if thread and thread.isAlive()]
+            threads = [thread.join(timeout=3.14) for thread in threads if thread and thread.is_alive()]
 
 
 def __handle_requests(reqs, suspicious_patterns, retry_protocol_mismatches, logger=logging.log):
