@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2013-2020 CERN
+# Copyright 2013-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,13 +18,14 @@
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2013-2018
 # - Vincent Garonne <vincent.garonne@cern.ch>, 2014-2018
 # - Wen Guan <wen.guan@cern.ch>, 2014-2016
-# - Martin Barisits <martin.barisits@cern.ch>, 2016-2020
+# - Martin Barisits <martin.barisits@cern.ch>, 2016-2021
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018
-# - maatthias <maatthias@gmail.com>, 2019
+# - Matt Snyder <msnyder@bnl.gov>, 2019-2021
 # - Brandon White <bjwhite@fnal.gov>, 2019
 # - Nick Smith <nick.smith@cern.ch>, 2020
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2020-2021
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
+# - Radu Carpa <radu.carpa@cern.ch>, 2021
 
 """
 Conveyor is a daemon to manage file transfers.
@@ -63,11 +64,12 @@ graceful_stop = threading.Event()
 
 datetime.datetime.strptime('', '')
 
-TRANSFER_TOOL = config_get('conveyor', 'transfertool', False, None)
+TRANSFER_TOOL = config_get('conveyor', 'transfertool', False, None)  # NOTE: This should eventually be completely removed, as it can be fetched from the request
+FILTER_TRANSFERTOOL = config_get('conveyor', 'filter_transfertool', False, None)  # NOTE: TRANSFERTOOL to filter requests on
 
 
 def poller(once=False, activities=None, sleep_time=60,
-           fts_bulk=100, db_bulk=1000, older_than=60, activity_shares=None):
+           fts_bulk=100, db_bulk=1000, older_than=60, activity_shares=None, partition_wait_time=10):
     """
     Main loop to check the status of a transfer primitive with a transfertool.
     """
@@ -85,6 +87,8 @@ def poller(once=False, activities=None, sleep_time=60,
     if activity_shares:
         activities.sort()
         executable += '--activity_shares' + str(activity_shares)
+    if FILTER_TRANSFERTOOL:
+        executable += ' --filter-transfertool ' + FILTER_TRANSFERTOOL
 
     hostname = socket.getfqdn()
     pid = os.getpid()
@@ -95,7 +99,8 @@ def poller(once=False, activities=None, sleep_time=60,
     logger = formatted_logger(logging.log, prefix + '%s')
     logger(logging.INFO, 'Poller starting - db_bulk (%i) fts_bulk (%i) timeout (%s)' % (db_bulk, fts_bulk, timeout))
 
-    time.sleep(10)  # To prevent running on the same partition if all the poller restart at the same time
+    if partition_wait_time:
+        time.sleep(partition_wait_time)  # To prevent running on the same partition if all the poller restart at the same time
     heart_beat = heartbeat.live(executable, hostname, pid, hb_thread)
     prefix = 'conveyor-poller[%i/%i] : ' % (heart_beat['assign_thread'], heart_beat['nr_threads'])
     logger = formatted_logger(logging.log, prefix + '%s')
@@ -119,7 +124,7 @@ def poller(once=False, activities=None, sleep_time=60,
                     continue
 
                 start_time = time.time()
-                logger(logging.DEBUG, 'Start to poll transfers older than %i seconds for activity %s' % (older_than, activity))
+                logger(logging.DEBUG, 'Start to poll transfers older than %i seconds for activity %s using transfer tool: %s' % (older_than, activity, FILTER_TRANSFERTOOL))
                 transfs = request_core.get_next(request_type=[RequestType.TRANSFER, RequestType.STAGEIN, RequestType.STAGEOUT],
                                                 state=[RequestState.SUBMITTED],
                                                 limit=db_bulk,
@@ -127,7 +132,8 @@ def poller(once=False, activities=None, sleep_time=60,
                                                 total_workers=heart_beat['nr_threads'], worker_number=heart_beat['assign_thread'],
                                                 mode_all=False, hash_variable='id',
                                                 activity=activity,
-                                                activity_shares=activity_shares)
+                                                activity_shares=activity_shares,
+                                                transfertool=FILTER_TRANSFERTOOL)
 
                 record_timer('daemons.conveyor.poller.000-get_next', (time.time() - start_time) * 1000)
 
@@ -222,7 +228,7 @@ def run(once=False, sleep_time=60, activities=None,
 
         # Interruptible joins require a timeout.
         while threads:
-            threads = [thread.join(timeout=3.14) for thread in threads if thread and thread.isAlive()]
+            threads = [thread.join(timeout=3.14) for thread in threads if thread and thread.is_alive()]
 
 
 def poll_transfers(external_host, xfers, request_ids=None, timeout=None, logger=logging.log):
