@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2015-2020 CERN
+# Copyright 2015-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,15 +15,16 @@
 #
 # Authors:
 # - Wen Guan <wen.guan@cern.ch>, 2015-2016
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2015
-# - Martin Barisits <martin.barisits@cern.ch>, 2015-2018
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2015-2021
+# - Martin Barisits <martin.barisits@cern.ch>, 2015-2021
 # - Vincent Garonne <vincent.garonne@cern.ch>, 2015-2018
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2018
 # - Robert Illingworth <illingwo@fnal.gov>, 2018
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2020-2021
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
+# - Sahan Dilshan <32576163+sahandilshan@users.noreply.github.com>, 2021
 
 """
 Conveyor is a daemon to manage file transfers.
@@ -65,17 +66,17 @@ class Receiver(object):
         self.__total_threads = total_threads
         self.__full_mode = full_mode
 
-    def on_error(self, headers, message):
+    def on_error(self, frame):
         record_counter('daemons.conveyor.receiver.error')
-        logging.error('[%s] %s' % (self.__broker, message))
+        logging.error('[%s] %s' % (self.__broker, frame.body))
 
-    def on_message(self, headers, message):
+    def on_message(self, frame):
         record_counter('daemons.conveyor.receiver.message_all')
 
         try:
-            msg = json.loads(message)
+            msg = json.loads(frame.body)
         except Exception:
-            msg = json.loads(message[:-1])  # Note: I am not sure if this is needed anymore, this was due to an unparsable EOT character
+            msg = json.loads(frame.body[:-1])  # Note: I am not sure if this is needed anymore, this was due to an unparsable EOT character
 
         if 'vo' not in msg or msg['vo'] != get_policy():
             return
@@ -249,7 +250,6 @@ def receiver(id, total_threads=1, full_mode=False):
                 record_counter('daemons.messaging.fts3.reconnect.%s' % conn.transport._Transport__host_and_ports[0][0].split('.')[0])
 
                 conn.set_listener('rucio-messaging-fts3', Receiver(broker=conn.transport._Transport__host_and_ports[0], id=id, total_threads=total_threads, full_mode=full_mode))
-                conn.start()
                 if not use_ssl:
                     conn.connect(username, password, wait=True)
                 else:
@@ -301,4 +301,4 @@ def run(once=False, total_threads=1, full_mode=False):
 
     # Interruptible joins require a timeout.
     while threads:
-        threads = [thread.join(timeout=3.14) for thread in threads if thread and thread.isAlive()]
+        threads = [thread.join(timeout=3.14) for thread in threads if thread and thread.is_alive()]
