@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2013-2020 CERN
+# Copyright 2013-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,19 +18,20 @@
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2013-2019
 # - Ralph Vigne <ralph.vigne@cern.ch>, 2013
 # - Vincent Garonne <vincent.garonne@cern.ch>, 2014-2018
-# - Martin Barisits <martin.barisits@cern.ch>, 2014-2020
+# - Martin Barisits <martin.barisits@cern.ch>, 2014-2021
 # - Wen Guan <wen.guan@cern.ch>, 2014-2016
 # - Tomáš Kouba <tomas.kouba@cern.ch>, 2014
 # - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2016
 # - dciangot <diego.ciangottini@cern.ch>, 2018
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - maatthias <maatthias@gmail.com>, 2019
+# - Matt Snyder <msnyder@bnl.gov>, 2019-2021
 # - Brandon White <bjwhite@fnal.gov>, 2019
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2020-2021
 # - Nick Smith <nick.smith@cern.ch>, 2020
 # - James Perry <j.perry@epcc.ed.ac.uk>, 2020
 # - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
+# - Radu Carpa <radu.carpa@cern.ch>, 2021
 
 """
 Conveyor transfer submitter is a daemon to manage non-tape file transfers.
@@ -67,15 +68,17 @@ except Exception:
 graceful_stop = threading.Event()
 
 USER_TRANSFERS = config_get('conveyor', 'user_transfers', False, None)
-TRANSFER_TOOL = config_get('conveyor', 'transfertool', False, None)
+TRANSFER_TOOL = config_get('conveyor', 'transfertool', False, None)  # NOTE: This should eventually be completely removed, as it can be fetched from the request
+FILTER_TRANSFERTOOL = config_get('conveyor', 'filter_transfertool', False, None)  # NOTE: TRANSFERTOOL to filter requests on
 TRANSFER_TYPE = config_get('conveyor', 'transfertype', False, 'single')
 
 GET_TRANSFERS_COUNTER = Counter('rucio_daemons_conveyor_submitter_get_transfers', 'Number of transfers retrieved')
 
 
-def submitter(once=False, rses=None, mock=False,
+def submitter(once=False, rses=None, partition_wait_time=10,
               bulk=100, group_bulk=1, group_policy='rule', source_strategy=None,
-              activities=None, sleep_time=600, max_sources=4, retry_other_fts=False):
+              activities=None, sleep_time=600, max_sources=4, retry_other_fts=False,
+              filter_transfertool=FILTER_TRANSFERTOOL, transfertool=TRANSFER_TOOL, transfertype=TRANSFER_TYPE):
     """
     Main loop to submit a new transfer primitive to a transfertool.
     """
@@ -118,6 +121,8 @@ def submitter(once=False, rses=None, mock=False,
     if activities:
         activities.sort()
         executable += '--activities ' + str(activities)
+    if filter_transfertool:
+        executable += ' --filter-transfertool ' + filter_transfertool
 
     hostname = socket.getfqdn()
     pid = os.getpid()
@@ -128,7 +133,8 @@ def submitter(once=False, rses=None, mock=False,
     logger = formatted_logger(logging.log, prefix + '%s')
     logger(logging.INFO, 'Submitter starting with timeout %s', timeout)
 
-    time.sleep(10)  # To prevent running on the same partition if all the poller restart at the same time
+    if partition_wait_time:
+        time.sleep(partition_wait_time)  # To prevent running on the same partition if all the poller restart at the same time
     heart_beat = heartbeat.live(executable, hostname, pid, hb_thread)
     prefix = 'conveyor-submitter[%i/%i] : ' % (heart_beat['assign_thread'], heart_beat['nr_threads'])
     logger = formatted_logger(logging.log, prefix + '%s')
@@ -166,11 +172,12 @@ def submitter(once=False, rses=None, mock=False,
                                             activity=activity,
                                             rses=rse_ids,
                                             schemes=scheme,
-                                            mock=mock,
                                             max_sources=max_sources,
                                             bring_online=bring_online,
                                             retry_other_fts=retry_other_fts,
+                                            transfertool=filter_transfertool,
                                             logger=logger)
+
                 record_timer('daemons.conveyor.transfer_submitter.get_transfers.per_transfer', (time.time() - start_time) * 1000 / (len(transfers) if transfers else 1))
                 record_counter('daemons.conveyor.transfer_submitter.get_transfers', len(transfers))
                 GET_TRANSFERS_COUNTER.inc(len(transfers))
@@ -181,26 +188,26 @@ def submitter(once=False, rses=None, mock=False,
                 logger(logging.INFO, 'Starting to group transfers for %s', activity)
                 start_time = time.time()
 
-                grouped_jobs = bulk_group_transfer(transfers, group_policy, group_bulk, source_strategy, max_time_in_queue)
+                grouped_jobs = bulk_group_transfer(transfers, group_policy, group_bulk, source_strategy, max_time_in_queue, group_by_scope=user_transfer)
                 record_timer('daemons.conveyor.transfer_submitter.bulk_group_transfer', (time.time() - start_time) * 1000 / (len(transfers) if transfers else 1))
 
                 logger(logging.INFO, 'Starting to submit transfers for %s', activity)
 
-                if TRANSFER_TOOL in ['fts3', 'mock']:
+                if transfertool in ['fts3', 'mock']:
                     for external_host in grouped_jobs:
                         if not user_transfer:
                             for job in grouped_jobs[external_host]:
                                 # submit transfers
                                 submit_transfer(external_host=external_host, job=job, submitter='transfer_submitter',
-                                                timeout=timeout, logger=logger)
+                                                timeout=timeout, logger=logger, transfertool=transfertool)
                         else:
                             for _, jobs in iteritems(grouped_jobs[external_host]):
                                 # submit transfers
                                 for job in jobs:
                                     submit_transfer(external_host=external_host, job=job, submitter='transfer_submitter',
-                                                    timeout=timeout, user_transfer_job=user_transfer, logger=logger)
-                elif TRANSFER_TOOL == 'globus':
-                    if TRANSFER_TYPE == 'bulk':
+                                                    timeout=timeout, user_transfer_job=user_transfer, logger=logger, transfertool=transfertool)
+                elif transfertool == 'globus':
+                    if transfertype == 'bulk':
                         # build bulk job file list per external host to send to submit_transfer
                         for external_host in grouped_jobs:
                             # pad the job with job_params; irrelevant for globus but needed for further rucio parsing
@@ -208,7 +215,8 @@ def submitter(once=False, rses=None, mock=False,
                             for job in grouped_jobs[external_host]:
                                 submitjob.get('files').append(job.get('files')[0])
                             logger(logging.DEBUG, 'submitjob: %s' % submitjob)
-                            submit_transfer(external_host=external_host, job=submitjob, submitter='transfer_submitter', timeout=timeout, logger=logger)
+                            submit_transfer(external_host=external_host, job=submitjob, submitter='transfer_submitter',
+                                            timeout=timeout, logger=logger, transfertool=transfertool)
                     else:
                         # build single job files and individually send to submit_transfer
                         job_params = grouped_jobs[''][0].get('job_params') if grouped_jobs else None
@@ -217,7 +225,8 @@ def submitter(once=False, rses=None, mock=False,
                                 for file in job['files']:
                                     singlejob = {'files': [file], 'job_params': job_params}
                                     logger(logging.DEBUG, 'singlejob: %s' % singlejob)
-                                    submit_transfer(external_host=external_host, job=singlejob, submitter='transfer_submitter', timeout=timeout, logger=logger)
+                                    submit_transfer(external_host=external_host, job=singlejob, submitter='transfer_submitter',
+                                                    timeout=timeout, logger=logger, transfertool=transfertool)
                 else:
                     logger(logging.ERROR, 'Unknown transfer tool')
 
@@ -257,9 +266,6 @@ def run(once=False, group_bulk=1, group_policy='rule', mock=False,
     if rucio.db.sqla.util.is_old_db():
         raise exception.DatabaseException('Database was not updated, daemon won\'t start')
 
-    if mock:
-        logging.info('mock source replicas: enabled')
-
     multi_vo = config_get_bool('common', 'multi_vo', raise_exception=False, default=False)
     working_rses = None
     if rses or include_rses or exclude_rses:
@@ -296,7 +302,6 @@ def run(once=False, group_bulk=1, group_policy='rule', mock=False,
                                                           'group_bulk': group_bulk,
                                                           'group_policy': group_policy,
                                                           'activities': activities,
-                                                          'mock': mock,
                                                           'sleep_time': sleep_time,
                                                           'max_sources': max_sources,
                                                           'source_strategy': source_strategy,
@@ -308,12 +313,12 @@ def run(once=False, group_bulk=1, group_policy='rule', mock=False,
 
     # Interruptible joins require a timeout.
     while threads:
-        threads = [thread.join(timeout=3.14) for thread in threads if thread and thread.isAlive()]
+        threads = [thread.join(timeout=3.14) for thread in threads if thread and thread.is_alive()]
 
 
 def __get_transfers(total_workers=0, worker_number=0, failover_schemes=None, limit=None, activity=None, older_than=None,
-                    rses=None, schemes=None, mock=False, max_sources=4, bring_online=43200,
-                    retry_other_fts=False, logger=logging.log):
+                    rses=None, schemes=None, max_sources=4, bring_online=43200,
+                    retry_other_fts=False, transfertool=None, logger=logging.log):
     """
     Get transfers to process
 
@@ -325,11 +330,11 @@ def __get_transfers(total_workers=0, worker_number=0, failover_schemes=None, lim
     :param older_than:       Only select requests older than this DateTime.
     :param rses:             List of rse_id to select requests.
     :param schemes:          Schemes to process.
-    :param mock:             Mock testing.
     :param max_sources:      Max sources.
-    :bring_online:           Bring online timeout.
+    :param bring_online:     Bring online timeout.
     :param logger:           Optional decorated logger that can be passed from the calling daemons or servers.
-    :retry_other_fts:        Retry other fts servers if needed
+    :param retry_other_fts:  Retry other fts servers if needed
+    :param transfertool:     The transfer tool as specified in rucio.cfg
     :returns:                List of transfers
     """
 
@@ -342,20 +347,18 @@ def __get_transfers(total_workers=0, worker_number=0, failover_schemes=None, lim
                                                                                                                                      schemes=schemes,
                                                                                                                                      bring_online=bring_online,
                                                                                                                                      retry_other_fts=retry_other_fts,
-                                                                                                                                     failover_schemes=failover_schemes)
-    request_core.set_requests_state(reqs_no_source, RequestState.NO_SOURCES, logger=logger)
-    request_core.set_requests_state(reqs_only_tape_source, RequestState.ONLY_TAPE_SOURCES, logger=logger)
-    request_core.set_requests_state(reqs_scheme_mismatch, RequestState.MISMATCH_SCHEME, logger=logger)
+                                                                                                                                     failover_schemes=failover_schemes,
+                                                                                                                                     transfertool=transfertool)
+    request_core.set_requests_state_if_possible(reqs_no_source, RequestState.NO_SOURCES, logger=logger)
+    request_core.set_requests_state_if_possible(reqs_only_tape_source, RequestState.ONLY_TAPE_SOURCES, logger=logger)
+    request_core.set_requests_state_if_possible(reqs_scheme_mismatch, RequestState.MISMATCH_SCHEME, logger=logger)
 
     for request_id in transfers:
         sources = transfers[request_id]['sources']
         sources = __sort_ranking(sources, logger=logger)
         if len(sources) > max_sources:
             sources = sources[:max_sources]
-        if not mock:
-            transfers[request_id]['sources'] = sources
-        else:
-            transfers[request_id]['sources'] = __mock_sources(sources)
+        transfers[request_id]['sources'] = sources
 
         # remove link_ranking in the final sources
         sources = transfers[request_id]['sources']
@@ -422,18 +425,3 @@ def __sort_ranking(sources, logger=logging.log):
         ret_sources = ret_sources + sources_list
     logger(logging.DEBUG, "Sources after sorting: %s", str(ret_sources))
     return ret_sources
-
-
-def __mock_sources(sources):
-    """
-    Create mock sources
-
-    :param sources:  List of sources
-    :return:         List of mock sources
-    """
-
-    tmp_sources = []
-    for source in sources:
-        tmp_sources.append((source[0], ':'.join(['mock'] + source[1].split(':')[1:]), source[2], source[3]))
-    sources = tmp_sources
-    return tmp_sources
