@@ -52,6 +52,7 @@ from rucio.core.message import add_message
 from rucio.core.quarantined_replica import (list_quarantined_replicas,
                                             delete_quarantined_replicas,
                                             list_rses)
+import rucio.core.rse as rse_core
 from rucio.core.rse_expression_parser import parse_expression
 from rucio.core.vo import list_vos
 from rucio.rse import rsemanager as rsemgr
@@ -61,11 +62,11 @@ logging.getLogger("requests").setLevel(logging.CRITICAL)
 GRACEFUL_STOP = threading.Event()
 
 
-def reaper(rses=[], worker_number=0, total_workers=1, chunk_size=100, once=False, scheme=None):
+def reaper(rses, worker_number=0, total_workers=1, chunk_size=100, once=False, scheme=None):
     """
     Main loop to select and delete files.
 
-    :param rses: List of RSEs the reaper should work against. If empty, it considers all RSEs.
+    :param rses: List of RSEs the reaper should work against.
     :param worker_number: The worker number.
     :param total_workers:  The total number of workers.
     :param chunk_size: the size of chunk for deletion.
@@ -90,8 +91,9 @@ def reaper(rses=[], worker_number=0, total_workers=1, chunk_size=100, once=False
                          .format(locals()))
             nothing_to_do = True
 
-            random.shuffle(rses)
-            for rse_id in rses:
+            rses_to_process = list(set(rses) & set(list_rses()))
+            random.shuffle(rses_to_process)
+            for rse_id in rses_to_process:
                 replicas = list_quarantined_replicas(rse_id=rse_id,
                                                      limit=chunk_size, worker_number=worker_number,
                                                      total_workers=total_workers)
@@ -232,7 +234,7 @@ def run(total_workers=1, chunk_size=100, once=False, rses=[], scheme=None,
 
     all_rses = []
     for vo in vos:
-        all_rses.extend(list_rses(filters={'vo': vo}))
+        all_rses.extend([rse['id'] for rse in rse_core.list_rses(filters={'vo': vo})])
 
     if rses:
         invalid = set(rses) - set([rse['rse'] for rse in all_rses])
