@@ -15,7 +15,7 @@
 #
 # Authors:
 # - Vincent Garonne <vincent.garonne@cern.ch>, 2016-2018
-# - Martin Barisits <martin.barisits@cern.ch>, 2016-2020
+# - Martin Barisits <martin.barisits@cern.ch>, 2016-2021
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2016-2021
 # - Wen Guan <wguan.icedew@gmail.com>, 2016
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
@@ -26,8 +26,9 @@
 # - James Perry <j.perry@epcc.ed.ac.uk>, 2019
 # - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
 # - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
+# - Radu Carpa <radu.carpa@cern.ch>, 2021
 
 '''
 Reaper is a daemon to manage file deletion.
@@ -90,7 +91,7 @@ def get_rses_to_process(rses, include_rses, exclude_rses, vos):
     :param exclude_rses:       RSE expression to exclude RSEs from the Reaper.
     :param include_rses:       RSE expression to include RSEs.
     :param vos:                VOs on which to look for RSEs. Only used in multi-VO mode.
-                               If None, we either use all VOs if run from "def",
+                               If None, we either use all VOs if run from "def"
 
     :returns: A list of RSEs to process
     """
@@ -109,9 +110,11 @@ def get_rses_to_process(rses, include_rses, exclude_rses, vos):
             vos = [v['vo'] for v in list_vos()]
         logging.log(logging.INFO, 'Reaper: This instance will work on VO%s: %s' % ('s' if len(vos) > 1 else '', ', '.join([v for v in vos])))
 
-    cache_key = 'rses_to_process'
+    pid = os.getpid()
+    cache_key = 'rses_to_process_%s' % pid
     if multi_vo:
         cache_key += '@%s' % '-'.join(vo for vo in vos)
+
     result = REGION.get(cache_key)
     if result is not NO_VALUE:
         return result
@@ -237,7 +240,7 @@ def get_rses_to_hostname_mapping():
     """
     Return a dictionaries mapping the RSEs to the hostname of the SE
 
-    :returns: Dictionary with RSE_id as key and (hostname, rse_info) as value
+    :returns:      Dictionary with RSE_id as key and (hostname, rse_info) as value
     """
 
     result = REGION.get('rse_hostname_mapping')
@@ -245,7 +248,12 @@ def get_rses_to_hostname_mapping():
         result = {}
         all_rses = list_rses()
         for rse in all_rses:
-            rse_protocol = get_rse_protocols(rse_id=rse['id'])
+            try:
+                rse_protocol = get_rse_protocols(rse_id=rse['id'])
+            except RSENotFound:
+                logging.log(logging.WARNING, 'RSE deleted while constructing rse-to-hostname mapping. Skipping %s', rse['rse'])
+                continue
+
             for prot in rse_protocol['protocols']:
                 if prot['domains']['wan']['delete'] == 1:
                     result[rse['id']] = (prot['hostname'], rse_protocol)
@@ -529,13 +537,11 @@ def reaper(rses, include_rses, exclude_rses, vos=None, chunk_size=100, once=Fals
                             tot_threads_for_hostname += payload_cnt[key]
                         if key.split(',')[0] == str(rse_id):
                             tot_threads_for_rse += payload_cnt[key]
-
                 max_deletion_thread = get_max_deletion_threads_by_hostname(rse_hostname)
                 if rse_hostname_key in payload_cnt and tot_threads_for_hostname >= max_deletion_thread:
                     logger(logging.DEBUG, 'Too many deletion threads for %s on RSE %s. Back off', rse_hostname, rse_name)
                     # Might need to reschedule a try on this RSE later in the same cycle
                     continue
-
                 logger(logging.INFO, 'Nb workers on %s smaller than the limit (current %i vs max %i). Starting new worker on RSE %s', rse_hostname, tot_threads_for_hostname, max_deletion_thread, rse_name)
                 live(executable, hostname, pid, hb_thread, older_than=600, hash_executable=None, payload=rse_hostname_key, session=None)
                 logger(logging.DEBUG, 'Total deletion workers for %s : %i', rse_hostname, tot_threads_for_hostname + 1)
@@ -562,7 +568,6 @@ def reaper(rses, include_rses, exclude_rses, vos=None, chunk_size=100, once=Fals
                     continue
                 except Exception:
                     logger(logging.CRITICAL, 'Exception', exc_info=True)
-
                 # Physical  deletion will take place there
                 try:
                     prot = rsemgr.create_protocol(rse_info, 'delete', scheme=scheme, logger=logger)
@@ -683,4 +688,4 @@ def run(threads=1, chunk_size=100, once=False, greedy=False, rses=None, scheme=N
 
     # Interruptible joins require a timeout.
     while threads_list:
-        threads_list = [thread.join(timeout=3.14) for thread in threads_list if thread and thread.isAlive()]
+        threads_list = [thread.join(timeout=3.14) for thread in threads_list if thread and thread.is_alive()]
