@@ -1,4 +1,5 @@
-# Copyright 2012-2020 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2012-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,10 +15,10 @@
 #
 # Authors:
 # - Vincent Garonne <vincent.garonne@cern.ch>, 2012-2018
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2012-2019
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2012-2021
 # - Angelos Molfetas <Angelos.Molfetas@cern.ch>, 2012
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2012
-# - Joaquin Bogado <jbogado@linti.unlp.edu.ar>, 2014-2018
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2012-2021
+# - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2014-2018
 # - Cheng-Hsi Chao <cheng-hsi.chao@cern.ch>, 2014
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2015
 # - Martin Barisits <martin.barisits@cern.ch>, 2015-2019
@@ -25,18 +26,20 @@
 # - Tobias Wegner <twegner@cern.ch>, 2018
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
 # - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
 # - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
-#
-# PY3K COMPATIBLE
+# - Tomas Javurek <tomas.javurek@cern.ch>, 2020
 
 from __future__ import print_function
 
+import os
 import re
 import unittest
 from os import remove, unlink, listdir, rmdir, stat, path, environ
 
+import pytest
+
 from rucio.client.accountlimitclient import AccountLimitClient
 from rucio.client.didclient import DIDClient
 from rucio.client.replicaclient import ReplicaClient
@@ -45,7 +48,6 @@ from rucio.client.ruleclient import RuleClient
 from rucio.common.config import config_get, config_get_bool
 from rucio.common.types import InternalScope, InternalAccount
 from rucio.common.utils import generate_uuid, get_tmp_dir, md5, render_json
-from rucio.daemons.abacus import account as abacus_account
 from rucio.rse import rsemanager as rsemgr
 from rucio.tests.common import execute, account_name_generator, rse_name_generator, file_generator, scope_name_generator
 
@@ -323,10 +325,12 @@ class TestBinRucio(unittest.TestCase):
         if environ.get('SUITE', 'all') != 'client':
             from rucio.db.sqla import session, models
             db_session = session.get_session()
-            db_session.query(models.RSEFileAssociation).filter_by(name=tmp_file1_name, scope=InternalScope(self.user, **self.vo)).delete()
-            db_session.query(models.ReplicaLock).delete()
-            db_session.query(models.ReplicationRule).filter_by(name=tmp_file1_name, scope=InternalScope(self.user, **self.vo)).delete()
-            db_session.query(models.DataIdentifier).filter_by(name=tmp_file1_name, scope=InternalScope(self.user, **self.vo)).delete()
+            internal_scope = InternalScope(self.user, **self.vo)
+            db_session.query(models.RSEFileAssociation).filter_by(name=tmp_file1_name, scope=internal_scope).delete()
+            db_session.query(models.ReplicaLock).filter_by(name=tmp_file1_name, scope=internal_scope).delete()
+            db_session.query(models.ReplicationRule).filter_by(name=tmp_file1_name, scope=internal_scope).delete()
+            db_session.query(models.DidMeta).filter_by(name=tmp_file1_name, scope=internal_scope).delete()
+            db_session.query(models.DataIdentifier).filter_by(name=tmp_file1_name, scope=internal_scope).delete()
             db_session.commit()
             tmp_file4 = file_generator()
             checksum_tmp_file4 = md5(tmp_file4)
@@ -335,7 +339,7 @@ class TestBinRucio(unittest.TestCase):
             exitcode, out, err = execute(cmd)
             print(out)
             print(err)
-            assert (self.upload_success_str % path.basename(tmp_file4)) in out
+            assert (self.upload_success_str % path.basename(tmp_file4)) in out or (self.upload_success_str % path.basename(tmp_file4)) in err
             assert checksum_tmp_file4 == [replica for replica in self.replica_client.list_replicas(dids=[{'name': tmp_file1_name, 'scope': self.user}])][0]['md5']
 
             # try to upload file that already exists on RSE and is already registered -> no overwrite
@@ -345,7 +349,7 @@ class TestBinRucio(unittest.TestCase):
             print(out)
             print(err)
             remove(tmp_file4)
-            assert 'File already registered' in out
+            assert 'File already registered' in out or 'File already registered' in err
 
     def test_upload_file_guid(self):
         """CLIENT(USER): Rucio upload file with guid"""
@@ -373,7 +377,7 @@ class TestBinRucio(unittest.TestCase):
         print(out)
         print(err)
         # get the rule for the file
-        cmd = "rucio list-rules {0}:{1} | grep {0}:{1} | cut -f1 -d\ ".format(self.user, tmp_file1_name)  # NOQA: W605
+        cmd = r"rucio list-rules {0}:{1} | grep {0}:{1} | cut -f1 -d\ ".format(self.user, tmp_file1_name)
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, err)
@@ -581,6 +585,7 @@ class TestBinRucio(unittest.TestCase):
         except Exception:
             pass
 
+    @pytest.mark.noparallel(reason='fails when run in parallel')
     def test_download_no_subdir(self):
         """CLIENT(USER): Rucio download files with --no-subdir and check that files already found locally are not replaced"""
         tmp_file = file_generator()
@@ -726,6 +731,32 @@ class TestBinRucio(unittest.TestCase):
         print(out, err)
         assert re.search(tmp_file1[5:], out) is not None
 
+    def test_download_timeout_options_accepted(self):
+        """CLIENT(USER): Rucio download timeout options """
+        tmp_file1 = file_generator()
+        # add files
+        cmd = 'rucio upload --rse {0} --scope {1} {2}'.format(self.def_rse, self.user, tmp_file1)
+        print(self.marker + cmd)
+        exitcode, out, err = execute(cmd)
+        print(out, err)
+        # download files
+        cmd = 'rucio download --dir /tmp --transfer-timeout 3 --transfer-speed-timeout 1000 {0}:{1}'.format(self.user, tmp_file1[5:])  # triming '/tmp/' from filename
+        print(self.marker + cmd)
+        exitcode, out, err = execute(cmd)
+        print(out, err)
+        assert 'successfully downloaded' in err
+        # search for the files with ls
+        cmd = 'ls /tmp/'    # search in /tmp/
+        print(self.marker + cmd)
+        exitcode, out, err = execute(cmd)
+        print(out, err)
+
+        # Check that PFN the transfer-speed-timeout option is not accepted for --pfn
+        cmd = 'rucio -v download --rse {0} --transfer-speed-timeout 1 --pfn http://a.b.c/ {1}:{2}'.format(self.def_rse, self.user, tmp_file1)
+        exitcode, out, err = execute(cmd)
+        print(out, err)
+        assert "Download with --pfn doesn't support --transfer-speed-timeout" in err
+
     def test_download_metalink_file(self):
         """CLIENT(USER): Rucio download with metalink file"""
         metalink_file_path = generate_uuid()
@@ -734,11 +765,13 @@ class TestBinRucio(unittest.TestCase):
         # Use filter and metalink option
         cmd = 'rucio download --scope mock --filter size=1 --metalink=test'
         exitcode, out, err = execute(cmd)
+        print(out, err)
         assert 'Arguments filter and metalink cannot be used together' in err
 
         # Use did and metalink option
         cmd = 'rucio download --metalink=test mock:test'
         exitcode, out, err = execute(cmd)
+        print(out, err)
         assert 'Arguments dids and metalink cannot be used together' in err
 
         # Download only with metalink file
@@ -746,14 +779,17 @@ class TestBinRucio(unittest.TestCase):
         tmp_file_name = tmp_file[5:]
         cmd = 'rucio upload --rse {0} --scope {1} {2}'.format(self.def_rse, scope, tmp_file)
         exitcode, out, err = execute(cmd)
+        print(out, err)
         replica_file = ReplicaClient().list_replicas([{'scope': scope, 'name': tmp_file_name}], metalink=True)
         with open(metalink_file_path, 'w+') as metalink_file:
             metalink_file.write(replica_file)
         cmd = 'rucio download --dir /tmp --metalink {0}'.format(metalink_file_path)
         exitcode, out, err = execute(cmd)
+        print(out, err)
         remove(metalink_file_path)
         cmd = 'ls /tmp/{0}'.format(scope)
         exitcode, out, err = execute(cmd)
+        print(out, err)
         assert re.search(tmp_file_name, out) is not None
 
     def test_download_succeeds_md5only(self):
@@ -813,7 +849,7 @@ class TestBinRucio(unittest.TestCase):
         exitcode, out, err = execute(cmd)
         print(out, err)
 
-        report = 'Local\ checksum\:\ {0},\ Rucio\ checksum\:\ 0123456789abcdef0123456789abcdef'.format(file_md5)  # NOQA: W605
+        report = r'Local\ checksum\:\ {0},\ Rucio\ checksum\:\ 0123456789abcdef0123456789abcdef'.format(file_md5)
         print('searching', report, 'in', err)
         assert re.search(report, err) is not None
 
@@ -858,6 +894,61 @@ class TestBinRucio(unittest.TestCase):
         search = '{0} successfully downloaded'.format(tmp_file1[5:])  # triming '/tmp/' from filename
         assert re.search(search, err) is not None
 
+    def test_list_blacklisted_replicas(self):
+        """CLIENT(USER): Rucio list replicas"""
+        # add rse
+        tmp_rse = rse_name_generator()
+        cmd = 'rucio-admin rse add {0}'.format(tmp_rse)
+        print(self.marker + cmd)
+        exitcode, out, err = execute(cmd)
+        print(out, err)
+        cmd = 'rucio-admin rse add-protocol --hostname blacklistreplica --scheme file --prefix /rucio --port 0 --impl rucio.rse.protocols.posix.Default ' \
+              '--domain-json \'{"wan": {"read": 1, "write": 1, "delete": 1, "third_party_copy": 1}}\' %s' % tmp_rse
+        print(self.marker + cmd)
+        exitcode, out, err = execute(cmd)
+        print(out, err)
+
+        # add files
+        tmp_file1 = file_generator()
+        file_name = tmp_file1[5:]  # triming '/tmp/' from filename
+        cmd = 'rucio upload --rse {0} --scope {1} {2}'.format(tmp_rse, self.user, tmp_file1)
+        print(self.marker + cmd)
+        exitcode, out, err = execute(cmd)
+        print(out, err)
+
+        # create dataset
+        tmp_dataset = self.user + ':DSet' + rse_name_generator()
+        cmd = 'rucio add-dataset ' + tmp_dataset
+        print(self.marker + cmd)
+        exitcode, out, err = execute(cmd)
+        print(out, err)
+        # add files to dataset
+        cmd = 'rucio attach {0} {1}:{2}'.format(tmp_dataset, self.user, file_name)
+        print(self.marker + cmd)
+        exitcode, out, err = execute(cmd)
+        print(out, err)
+
+        # Listing the replica should work before blacklisting the RSE
+        cmd = 'rucio list-file-replicas {}'.format(tmp_dataset)
+        print(self.marker + cmd)
+        exitcode, out, err = execute(cmd)
+        print(out, err)
+        assert tmp_rse in out
+
+        # Blacklist the rse
+        cmd = 'rucio-admin rse update --rse {} --setting availability_read --value False'.format(tmp_rse)
+        print(self.marker + cmd)
+        exitcode, out, err = execute(cmd)
+        print(out, err)
+        assert not err
+
+        # list-file-replicas should, by default, list replicas from blacklisted rses
+        cmd = 'rucio list-file-replicas {}'.format(tmp_dataset)
+        print(self.marker + cmd)
+        exitcode, out, err = execute(cmd)
+        print(out, err)
+        assert tmp_rse in out
+
     def test_create_rule(self):
         """CLIENT(USER): Rucio add rule"""
         tmp_file1 = file_generator()
@@ -936,18 +1027,18 @@ class TestBinRucio(unittest.TestCase):
         self.account_client.set_local_account_limit('root', tmp_rse, -1)
 
         # add rse atributes
-        cmd = 'rucio-admin rse set-attribute --rse {0} --key spacetoken --value ATLASSCRATCHDISK'.format(tmp_rse)
+        cmd = 'rucio-admin rse set-attribute --rse {0} --key spacetoken --value ATLASDELETERULE'.format(tmp_rse)
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, err)
         # add rules
-        cmd = "rucio add-rule {0}:{1} 1 'spacetoken=ATLASSCRATCHDISK'".format(self.user, tmp_file1[5:])
+        cmd = "rucio add-rule {0}:{1} 1 'spacetoken=ATLASDELETERULE'".format(self.user, tmp_file1[5:])
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(err)
         print(out)
         # get the rules for the file
-        cmd = "rucio list-rules {0}:{1} | grep {0}:{1} | cut -f1 -d\ ".format(self.user, tmp_file1[5:])  # NOQA: W605
+        cmd = r"rucio list-rules {0}:{1} | grep {0}:{1} | cut -f1 -d\ ".format(self.user, tmp_file1[5:])
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, err)
@@ -989,7 +1080,7 @@ class TestBinRucio(unittest.TestCase):
         exitcode, out, err = execute(cmd)
         print(out, err)
         # get the rule for the file
-        cmd = "rucio list-rules {0}:{1} | grep {0}:{1} | cut -f1 -d\ ".format(self.user, tmp_file1[5:])  # NOQA: W605
+        cmd = r"rucio list-rules {0}:{1} | grep {0}:{1} | cut -f1 -d\ ".format(self.user, tmp_file1[5:])
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, err)
@@ -1146,6 +1237,7 @@ class TestBinRucio(unittest.TestCase):
         print(err)
         assert re.search("Data identifier not found.", err) is not None
 
+    @pytest.mark.dirty
     def test_list_did_recursive(self):
         """ CLIENT(USER): List did recursive """
         # Setup nested collections
@@ -1176,6 +1268,7 @@ class TestBinRucio(unittest.TestCase):
         exitcode, out, err = execute(cmd)
         assert re.search("Option recursive cannot be used with wildcards", err) is not None
 
+    @pytest.mark.dirty
     def test_attach_many_dids(self):
         """ CLIENT(USER): Rucio attach many (>1000) DIDs """
         # Setup data for CLI check
@@ -1234,6 +1327,7 @@ class TestBinRucio(unittest.TestCase):
         # last file must be in the dataset
         assert re.search("{0}:{1}".format(self.user, files[-1]['name']), out) is not None
 
+    @pytest.mark.dirty
     def test_attach_many_dids_twice(self):
         """ CLIENT(USER): Attach many (>1000) DIDs twice """
         # Setup data for CLI check
@@ -1271,6 +1365,7 @@ class TestBinRucio(unittest.TestCase):
         exitcode, out, err = execute(cmd)
         assert re.search("{0}:{1}".format(self.user, new_dataset['name']), out) is not None
 
+    @pytest.mark.noparallel(reason='might override global RSE settings')
     def test_import_data(self):
         """ CLIENT(ADMIN): Import data into rucio"""
         file_path = 'data_import.json'
@@ -1284,14 +1379,18 @@ class TestBinRucio(unittest.TestCase):
         assert re.search('Data successfully imported', out) is not None
         remove(file_path)
 
+    @pytest.mark.noparallel(reason='fails when run in parallel')
     def test_export_data(self):
         """ CLIENT(ADMIN): Export data from rucio"""
         file_path = 'data_export.json'
         cmd = 'rucio-admin data export {0}'.format(file_path)
         exitcode, out, err = execute(cmd)
+        print(out, err)
         assert re.search('Data successfully exported', out) is not None
         remove(file_path)
 
+    @pytest.mark.dirty
+    @pytest.mark.noparallel(reason='fails when run in parallel')
     def test_set_tombstone(self):
         """ CLIENT(ADMIN): set a tombstone on a replica. """
         # Set tombstone on one replica
@@ -1317,6 +1416,7 @@ class TestBinRucio(unittest.TestCase):
         exitcode, out, err = execute(cmd)
         assert re.search('Replica not found', err) is not None
 
+    @pytest.mark.noparallel(reason='modifies account limit on pre-defined RSE')
     def test_list_account_limits(self):
         """ CLIENT (USER): list account limits. """
         rse = 'MOCK4'
@@ -1337,37 +1437,40 @@ class TestBinRucio(unittest.TestCase):
         self.account_client.set_local_account_limit(account, rse, -1)
         self.account_client.set_global_account_limit(account, rse_exp, -1)
 
+    @pytest.mark.noparallel(reason='modifies account limit on pre-defined RSE')
+    @pytest.mark.skipif('SUITE' in os.environ and os.environ['SUITE'] == 'client', reason='uses abacus daemon and core functions')
     def test_list_account_usage(self):
         """ CLIENT (USER): list account usage. """
-        if environ.get('SUITE', 'all') != 'client':
-            from rucio.db.sqla import session, models
-            from rucio.core.account_counter import increase
-            db_session = session.get_session()
-            db_session.query(models.AccountUsage).delete()
-            db_session.query(models.AccountLimit).delete()
-            db_session.query(models.AccountGlobalLimit).delete()
-            db_session.query(models.UpdatedAccountCounter).delete()
-            db_session.commit()
-            rse = 'MOCK4'
-            rse_id = self.rse_client.get_rse(rse)['id']
-            rse_exp = 'MOCK|MOCK4'
-            account = 'root'
-            usage = 4
-            local_limit = 10
-            local_left = local_limit - usage
-            global_limit = 20
-            global_left = global_limit - usage
-            self.account_client.set_local_account_limit(account, rse, local_limit)
-            self.account_client.set_global_account_limit(account, rse_exp, global_limit)
-            increase(rse_id, InternalAccount(account, **self.vo), 1, usage)
-            abacus_account.run(once=True)
-            cmd = 'rucio list-account-usage {0}'.format(account)
-            exitcode, out, err = execute(cmd)
-            assert re.search('.*{0}.*{1}.*{2}.*{3}'.format(rse, usage, local_limit, local_left), out) is not None
-            assert re.search('.*{0}.*{1}.*{2}.*{3}'.format(rse_exp, usage, global_limit, global_left), out) is not None
-            cmd = 'rucio list-account-usage --rse {0} {1}'.format(rse, account)
-            exitcode, out, err = execute(cmd)
-            assert re.search('.*{0}.*{1}.*{2}.*{3}'.format(rse, usage, local_limit, local_left), out) is not None
-            assert re.search('.*{0}.*{1}.*{2}.*{3}'.format(rse_exp, usage, global_limit, global_left), out) is not None
-            self.account_client.set_local_account_limit(account, rse, -1)
-            self.account_client.set_global_account_limit(account, rse_exp, -1)
+        from rucio.db.sqla import session, models
+        from rucio.core.account_counter import increase
+        from rucio.daemons.abacus import account as abacus_account
+
+        db_session = session.get_session()
+        db_session.query(models.AccountUsage).delete()
+        db_session.query(models.AccountLimit).delete()
+        db_session.query(models.AccountGlobalLimit).delete()
+        db_session.query(models.UpdatedAccountCounter).delete()
+        db_session.commit()
+        rse = 'MOCK4'
+        rse_id = self.rse_client.get_rse(rse)['id']
+        rse_exp = 'MOCK|MOCK4'
+        account = 'root'
+        usage = 4
+        local_limit = 10
+        local_left = local_limit - usage
+        global_limit = 20
+        global_left = global_limit - usage
+        self.account_client.set_local_account_limit(account, rse, local_limit)
+        self.account_client.set_global_account_limit(account, rse_exp, global_limit)
+        increase(rse_id, InternalAccount(account, **self.vo), 1, usage)
+        abacus_account.run(once=True)
+        cmd = 'rucio list-account-usage {0}'.format(account)
+        exitcode, out, err = execute(cmd)
+        assert re.search('.*{0}.*{1}.*{2}.*{3}'.format(rse, usage, local_limit, local_left), out) is not None
+        assert re.search('.*{0}.*{1}.*{2}.*{3}'.format(rse_exp, usage, global_limit, global_left), out) is not None
+        cmd = 'rucio list-account-usage --rse {0} {1}'.format(rse, account)
+        exitcode, out, err = execute(cmd)
+        assert re.search('.*{0}.*{1}.*{2}.*{3}'.format(rse, usage, local_limit, local_left), out) is not None
+        assert re.search('.*{0}.*{1}.*{2}.*{3}'.format(rse_exp, usage, global_limit, global_left), out) is not None
+        self.account_client.set_local_account_limit(account, rse, -1)
+        self.account_client.set_global_account_limit(account, rse_exp, -1)
