@@ -1,4 +1,5 @@
-# Copyright 2013-2018 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2013-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,25 +14,28 @@
 # limitations under the License.
 #
 # Authors:
-# - Vincent Garonne <vgaronne@gmail.com>, 2013-2018
-# - Martin Barisits <martin.barisits@cern.ch>, 2013-2018
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2013-2018
+# - Vincent Garonne <vincent.garonne@cern.ch>, 2013-2018
+# - Martin Barisits <martin.barisits@cern.ch>, 2013-2020
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2013-2020
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2013-2020
 # - Ralph Vigne <ralph.vigne@cern.ch>, 2013
 # - Yun-Pin Sun <winter0128@gmail.com>, 2013
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2013-2018
-# - Joaquin Bogado <jbogado@linti.unlp.edu.ar>, 2018
-# - Andrew Lister, <andrew.lister@stfc.ac.uk>, 2019
+# - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2018
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - Ruturaj Gujar, <ruturaj.gujar23@gmail.com>, 2019
-# - Eli Chadwick, <eli.chadwick@stfc.ac.uk>, 2020
+# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
+# - Ruturaj Gujar <ruturaj.gujar23@gmail.com>, 2019
+# - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
 # - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
+# - Jaroslav Guenther <jaroslav.guenther@cern.ch>, 2020
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
 
 from __future__ import print_function
 
+import unittest
 from datetime import datetime, timedelta
 
-from nose.tools import assert_equal, assert_not_equal, assert_raises, assert_true, assert_in, assert_not_in, raises
+import pytest
 
 from rucio.api import did
 from rucio.api import scope
@@ -41,6 +45,7 @@ from rucio.client.metaclient import MetaClient
 from rucio.client.replicaclient import ReplicaClient
 from rucio.client.rseclient import RSEClient
 from rucio.client.scopeclient import ScopeClient
+from rucio.common import exception
 from rucio.common.config import config_get, config_get_bool
 from rucio.common.exception import (DataIdentifierNotFound, DataIdentifierAlreadyExists,
                                     InvalidPath, KeyNotFound, UnsupportedOperation,
@@ -51,15 +56,15 @@ from rucio.core.account_limit import set_local_account_limit
 from rucio.core.did import (list_dids, add_did, delete_dids, get_did_atime, touch_dids, attach_dids, detach_dids,
                             get_metadata, set_metadata, get_did, get_did_access_cnt, add_did_to_followed,
                             get_users_following_did, remove_did_from_followed)
-from rucio.core.rse import get_rse_id
 from rucio.core.replica import add_replica
+from rucio.core.rse import get_rse_id
 from rucio.db.sqla.constants import DIDType
-
 from rucio.tests.common import rse_name_generator, scope_name_generator
 
 
-class TestDIDCore:
-    def setup(self):
+class TestDIDCore(unittest.TestCase):
+
+    def setUp(self):
         if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
             self.vo = {'vo': config_get('client', 'vo', raise_exception=False, default='tst')}
         else:
@@ -82,6 +87,7 @@ class TestDIDCore:
             add_did(scope=tmp_scope, name=dsn['name'], type='DATASET', account=root)
         delete_dids(dids=dsns, account=root)
 
+    @pytest.mark.dirty
     def test_touch_dids_atime(self):
         """ DATA IDENTIFIERS (CORE): Touch dids accessed_at timestamp"""
         tmp_scope = InternalScope('mock', **self.vo)
@@ -94,13 +100,14 @@ class TestDIDCore:
         now = datetime.utcnow()
 
         now -= timedelta(microseconds=now.microsecond)
-        assert_equal(None, get_did_atime(scope=tmp_scope, name=tmp_dsn1))
-        assert_equal(None, get_did_atime(scope=tmp_scope, name=tmp_dsn2))
+        assert get_did_atime(scope=tmp_scope, name=tmp_dsn1) is None
+        assert get_did_atime(scope=tmp_scope, name=tmp_dsn2) is None
 
         touch_dids(dids=[{'scope': tmp_scope, 'name': tmp_dsn1, 'type': DIDType.DATASET, 'accessed_at': now}])
-        assert_equal(now, get_did_atime(scope=tmp_scope, name=tmp_dsn1))
-        assert_equal(None, get_did_atime(scope=tmp_scope, name=tmp_dsn2))
+        assert now == get_did_atime(scope=tmp_scope, name=tmp_dsn1)
+        assert get_did_atime(scope=tmp_scope, name=tmp_dsn2) is None
 
+    @pytest.mark.dirty
     def test_touch_dids_access_cnt(self):
         """ DATA IDENTIFIERS (CORE): Increase dids access_cnt"""
         tmp_scope = InternalScope('mock', **self.vo)
@@ -111,14 +118,16 @@ class TestDIDCore:
         add_did(scope=tmp_scope, name=tmp_dsn1, type=DIDType.DATASET, account=root)
         add_did(scope=tmp_scope, name=tmp_dsn2, type=DIDType.DATASET, account=root)
 
-        assert_equal(None, get_did_access_cnt(scope=tmp_scope, name=tmp_dsn1))
-        assert_equal(None, get_did_access_cnt(scope=tmp_scope, name=tmp_dsn2))
+        assert get_did_access_cnt(scope=tmp_scope, name=tmp_dsn1) is None
+        assert get_did_access_cnt(scope=tmp_scope, name=tmp_dsn2) is None
 
         for i in range(100):
             touch_dids(dids=[{'scope': tmp_scope, 'name': tmp_dsn1, 'type': DIDType.DATASET}])
-        assert_equal(100, get_did_access_cnt(scope=tmp_scope, name=tmp_dsn1))
-        assert_equal(None, get_did_access_cnt(scope=tmp_scope, name=tmp_dsn2))
+        assert 100 == get_did_access_cnt(scope=tmp_scope, name=tmp_dsn1)
+        assert get_did_access_cnt(scope=tmp_scope, name=tmp_dsn2) is None
 
+    @pytest.mark.dirty
+    @pytest.mark.noparallel(reason='uses pre-defined RSE')
     def test_update_dids(self):
         """ DATA IDENTIFIERS (CORE): Update file size and checksum"""
         tmp_scope = InternalScope('mock', **self.vo)
@@ -133,14 +142,16 @@ class TestDIDCore:
         attach_dids(scope=tmp_scope, name=dsn, rse_id=get_rse_id(rse='MOCK', **self.vo), dids=files, account=root)
 
         set_metadata(scope=tmp_scope, name=lfn, key='adler32', value='0cc737ee')
-        assert_equal(get_metadata(scope=tmp_scope, name=lfn)['adler32'], '0cc737ee')
+        assert get_metadata(scope=tmp_scope, name=lfn)['adler32'] == '0cc737ee'
 
-        with assert_raises(DataIdentifierNotFound):
+        with pytest.raises(DataIdentifierNotFound):
             set_metadata(scope=tmp_scope, name='Nimportnawak', key='adler32', value='0cc737ee')
 
         set_metadata(scope=tmp_scope, name=lfn, key='bytes', value=724963577)
-        assert_equal(get_metadata(scope=tmp_scope, name=lfn)['bytes'], 724963577)
+        assert get_metadata(scope=tmp_scope, name=lfn)['bytes'] == 724963577
 
+    @pytest.mark.dirty
+    @pytest.mark.noparallel(reason='uses pre-defined RSE')
     def test_get_did_with_dynamic(self):
         """ DATA IDENTIFIERS (CORE): Get did with dynamic resolve of size"""
         tmp_scope = InternalScope('mock', **self.vo)
@@ -160,9 +171,11 @@ class TestDIDCore:
         add_did(scope=tmp_scope, name=tmp_dsn4, type=DIDType.CONTAINER, account=root)
         attach_dids(scope=tmp_scope, name=tmp_dsn4, dids=[{'scope': tmp_scope, 'name': tmp_dsn1}], account=root)
 
-        assert_equal(get_did(scope=tmp_scope, name=tmp_dsn1, dynamic=True)['bytes'], 20)
-        assert_equal(get_did(scope=tmp_scope, name=tmp_dsn4, dynamic=True)['bytes'], 20)
+        assert get_did(scope=tmp_scope, name=tmp_dsn1, dynamic=True)['bytes'] == 20
+        assert get_did(scope=tmp_scope, name=tmp_dsn4, dynamic=True)['bytes'] == 20
 
+    @pytest.mark.dirty
+    @pytest.mark.noparallel(reason='uses pre-defined RSE')
     def test_reattach_dids(self):
         """ DATA IDENTIFIERS (CORE): Repeatedly attach and detach DIDs """
         tmp_scope = InternalScope('mock', **self.vo)
@@ -183,6 +196,7 @@ class TestDIDCore:
 
         detach_dids(scope=tmp_scope, name=parent_name, dids=files)
 
+    @pytest.mark.dirty
     def test_add_did_to_followed(self):
         """ DATA IDENTIFIERS (CORE): Mark a did as followed """
         tmp_scope = InternalScope('mock', **self.vo)
@@ -196,8 +210,9 @@ class TestDIDCore:
         for user in users:
             rows += 1
 
-        assert_equal(rows, 1)
+        assert rows == 1
 
+    @pytest.mark.dirty
     def test_get_users_following_did(self):
         """ DATA IDENTIFIERS (CORE): Get the list of users following a did """
         tmp_scope = InternalScope('mock', **self.vo)
@@ -212,8 +227,9 @@ class TestDIDCore:
         for user in users:
             rows += 1
 
-        assert_equal(rows, 1)
+        assert rows == 1
 
+    @pytest.mark.dirty
     def test_remove_did_from_followed(self):
         """ DATA IDENTIFIERS (CORE): Mark a did as not followed """
         tmp_scope = InternalScope('mock', **self.vo)
@@ -228,7 +244,7 @@ class TestDIDCore:
         for user in users:
             rows += 1
 
-        assert_equal(rows, 1)
+        assert rows == 1
 
         remove_did_from_followed(scope=tmp_scope, name=dsn, account=root)
 
@@ -237,16 +253,17 @@ class TestDIDCore:
         for user in users:
             rows += 1
 
-        assert_equal(rows, 0)
+        assert rows == 0
 
 
-class TestDIDApi:
-    def setup(self):
+class TestDIDApi(unittest.TestCase):
+    def setUp(self):
         if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
             self.vo = {'vo': config_get('client', 'vo', raise_exception=False, default='tst')}
         else:
             self.vo = {}
 
+    @pytest.mark.dirty
     def test_list_new_dids(self):
         """ DATA IDENTIFIERS (API): List new identifiers """
         tmp_scope = scope_name_generator()
@@ -255,13 +272,14 @@ class TestDIDApi:
         for i in range(0, 5):
             did.add_did(scope=tmp_scope, name='%s-%i' % (tmp_dsn, i), type='DATASET', issuer='root', **self.vo)
         for i in did.list_new_dids('DATASET', **self.vo):
-            assert_not_equal(i, {})
-            assert_equal(str(i['did_type']), 'DATASET')
+            assert i != {}
+            assert i['did_type'] == DIDType.DATASET
             break
         for i in did.list_new_dids(**self.vo):
-            assert_not_equal(i, {})
+            assert i != {}
             break
 
+    @pytest.mark.dirty
     def test_update_new_dids(self):
         """ DATA IDENTIFIERS (API): List new identifiers and update the flag new """
         tmp_scope = scope_name_generator()
@@ -273,14 +291,14 @@ class TestDIDApi:
             did.add_did(scope=tmp_scope, name='%s-%i' % (tmp_dsn, i), type='DATASET', issuer='root', **self.vo)
             dids.append(d)
         st = did.set_new_dids(dids, None, **self.vo)
-        assert_true(st)
-        with assert_raises(DataIdentifierNotFound):
+        assert st
+        with pytest.raises(DataIdentifierNotFound):
             did.set_new_dids([{'scope': 'dummyscope', 'name': 'dummyname', 'did_type': DIDType.DATASET}], None, **self.vo)
 
 
-class TestDIDClients:
+class TestDIDClients(unittest.TestCase):
 
-    def setup(self):
+    def setUp(self):
         if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
             self.vo = {'vo': config_get('client', 'vo', raise_exception=False, default='tst')}
         else:
@@ -293,6 +311,8 @@ class TestDIDClients:
         self.replica_client = ReplicaClient()
         self.rse_client = RSEClient()
 
+    @pytest.mark.dirty
+    @pytest.mark.noparallel(reason='uses pre-defined RSE')
     def test_list_dids(self):
         """ DATA IDENTIFIERS (CLIENT): List dids by pattern."""
         tmp_scope = scope_name_generator()
@@ -309,28 +329,30 @@ class TestDIDClients:
         results = []
         for result in self.did_client.list_dids(tmp_scope, {'name': 'file_a_*'}, type='file'):
             results.append(result)
-        assert_equal(len(results), 2)
+        assert len(results) == 2
         results = []
         for result in self.did_client.list_dids(tmp_scope, {'name': 'file_a_1*'}, type='file'):
             results.append(result)
-        assert_equal(len(results), 1)
+        assert len(results) == 1
         results = []
         for result in self.did_client.list_dids(tmp_scope, {'name': 'file_*_1*'}, type='file'):
             results.append(result)
-        assert_equal(len(results), 2)
+        assert len(results) == 2
         results = []
         for result in self.did_client.list_dids(tmp_scope, {'name': 'file*'}, type='file'):
             results.append(result)
-        assert_equal(len(results), 3)
+        assert len(results) == 3
         results = []
 
         filters = {'name': 'file*', 'created_after': datetime.utcnow() - timedelta(hours=1)}
         for result in self.did_client.list_dids(tmp_scope, filters):
             results.append(result)
-        assert_equal(len(results), 0)
-        with assert_raises(UnsupportedOperation):
+        assert len(results) == 0
+        with pytest.raises(UnsupportedOperation):
             self.did_client.list_dids(tmp_scope, {'name': 'file*'}, type='whateverytype')
 
+    @pytest.mark.dirty
+    @pytest.mark.noparallel(reason='uses pre-defined scope names')
     def test_list_recursive(self):
         """ DATA IDENTIFIERS (CLIENT): List did recursive """
         # Create nested containers and datast
@@ -357,24 +379,26 @@ class TestDIDClients:
 
         # List DIDs not recursive - only the first container is expected
         dids = [str(did) for did in self.did_client.list_dids(scope=tmp_scope_1, recursive=False, type='all', filters={'name': tmp_container_1})]
-        assert_equal(dids, [tmp_container_1])
+        assert dids == [tmp_container_1]
 
         # List DIDs recursive - first container and all attached collections are expected
         dids = [str(did) for did in self.did_client.list_dids(scope=tmp_scope_1, recursive=True, type='all', filters={'name': tmp_container_1})]
-        assert_true(tmp_container_1 in dids)
-        assert_true(tmp_container_2 in dids)
-        assert_true(tmp_dataset_1 in dids)
-        assert_true(tmp_dataset_2 in dids)
-        assert_equal(len(dids), 4)
+        assert tmp_container_1 in dids
+        assert tmp_container_2 in dids
+        assert tmp_dataset_1 in dids
+        assert tmp_dataset_2 in dids
+        assert len(dids) == 4
 
         # List DIDs recursive - only containers are expected
         dids = [str(did) for did in self.did_client.list_dids(scope=tmp_scope_1, recursive=True, type='container', filters={'name': tmp_container_1})]
-        assert_true(tmp_container_1 in dids)
-        assert_true(tmp_container_2 in dids)
-        assert_true(tmp_dataset_1 not in dids)
-        assert_true(tmp_dataset_2 not in dids)
-        assert_equal(len(dids), 2)
+        assert tmp_container_1 in dids
+        assert tmp_container_2 in dids
+        assert tmp_dataset_1 not in dids
+        assert tmp_dataset_2 not in dids
+        assert len(dids) == 2
 
+    @pytest.mark.dirty
+    @pytest.mark.noparallel(reason='uses pre-defined scope, fails when run in parallel')
     def test_list_by_length(self):
         """ DATA IDENTIFIERS (CLIENT): List did with length """
         tmp_scope = 'mock'
@@ -386,20 +410,21 @@ class TestDIDClients:
         results = []
         for d in dids:
             results.append(d)
-        assert_not_equal(len(results), 0)
+        assert len(results) != 0
 
         dids = self.did_client.list_dids(tmp_scope, {'length.gt': -1, 'length.lt': 1})
         results = []
         for d in dids:
             results.append(d)
-        assert_equal(len(results), 0)
+        assert len(results) == 0
 
         dids = self.did_client.list_dids(tmp_scope, {'length': 0})
         results = []
         for d in dids:
             results.append(d)
-        assert_equal(len(results), 0)
+        assert len(results) == 0
 
+    @pytest.mark.dirty
     def test_list_by_metadata(self):
         """ DATA IDENTIFIERS (CLIENT): List did with metadata"""
         dsns = []
@@ -431,7 +456,7 @@ class TestDIDClients:
         for d in dids:
             results.append(d)
         for dsn in dsns:
-            assert_in(dsn, results)
+            assert dsn in results
         dsns.remove(tmp_dsn1)
 
         dids = self.did_client.list_dids(tmp_scope, {'project': 'data12_8TeV', 'run_number': 400001})
@@ -439,7 +464,7 @@ class TestDIDClients:
         for d in dids:
             results.append(d)
         for dsn in dsns:
-            assert_in(dsn, results)
+            assert dsn in results
         dsns.remove(tmp_dsn2)
 
         dids = self.did_client.list_dids(tmp_scope, {'project': 'data12_8TeV', 'stream_name': 'physics_Egamma', 'datatype': 'NTUP_SMWZ'})
@@ -447,11 +472,13 @@ class TestDIDClients:
         for d in dids:
             results.append(d)
         for dsn in dsns:
-            assert_in(dsn, results)
+            assert dsn in results
 
-        with assert_raises(KeyNotFound):
+        with pytest.raises(KeyNotFound):
             self.did_client.list_dids(tmp_scope, {'NotReallyAKey': 'NotReallyAValue'})
 
+    @pytest.mark.dirty
+    @pytest.mark.noparallel(reason='uses pre-defined RSE')
     def test_add_did(self):
         """ DATA IDENTIFIERS (CLIENT): Add, populate, list did content and create a sample"""
         tmp_scope = 'mock'
@@ -471,14 +498,14 @@ class TestDIDClients:
                         }
         rules = [{'copies': 1, 'rse_expression': 'MOCK', 'account': 'root'}]
 
-        with assert_raises(ScopeNotFound):
+        with pytest.raises(ScopeNotFound):
             self.did_client.add_dataset(scope='Nimportnawak', name=tmp_dsn, statuses={'monotonic': True}, meta=dataset_meta, rules=rules)
 
         files = [{'scope': InternalScope(tmp_scope, **self.vo), 'name': 'lfn.%(tmp_dsn)s.' % locals() + str(generate_uuid()), 'bytes': 724963570, 'adler32': '0cc737eb'}, ]
-        with assert_raises(DataIdentifierNotFound):
+        with pytest.raises(DataIdentifierNotFound):
             self.did_client.add_dataset(scope=tmp_scope, name=tmp_dsn, statuses={'monotonic': True}, meta=dataset_meta, rules=rules, files=files)
 
-        with assert_raises(DataIdentifierNotFound):
+        with pytest.raises(DataIdentifierNotFound):
             self.did_client.add_files_to_dataset(scope=tmp_scope, name=tmp_dsn, files=files)
 
         files = []
@@ -495,13 +522,13 @@ class TestDIDClients:
 
         rules = [{'copies': 1, 'rse_expression': 'CERN-PROD_TZERO', 'lifetime': timedelta(days=2), 'account': 'root'}]
 
-        with assert_raises(InvalidPath):
+        with pytest.raises(InvalidPath):
             self.did_client.add_dataset(scope=tmp_scope, name=tmp_dsn, statuses={'monotonic': True}, meta=dataset_meta, rules=rules, files=files, rse=tmp_rse)
 
         files_without_pfn = [{'scope': i['scope'], 'name': i['name'], 'bytes': i['bytes'], 'adler32': i['adler32'], 'meta': i['meta']} for i in files]
         self.did_client.add_dataset(scope=tmp_scope, name=tmp_dsn, statuses={'monotonic': True}, meta=dataset_meta, rules=rules, files=files_without_pfn, rse=tmp_rse)
 
-        with assert_raises(DataIdentifierAlreadyExists):
+        with pytest.raises(DataIdentifierAlreadyExists):
             self.did_client.add_dataset(scope=tmp_scope, name=tmp_dsn, files=files, rse=tmp_rse)
 
         files = []
@@ -517,7 +544,7 @@ class TestDIDClients:
                           'pfn': pfn, 'meta': file_meta})
         rules = [{'copies': 1, 'rse_expression': 'CERN-PROD_TZERO', 'lifetime': timedelta(days=2)}]
 
-        with assert_raises(InvalidPath):
+        with pytest.raises(InvalidPath):
             self.did_client.add_files_to_dataset(scope=tmp_scope, name=tmp_dsn, files=files, rse=tmp_rse)
         files_without_pfn = [{'scope': i['scope'], 'name': i['name'], 'bytes': i['bytes'], 'adler32': i['adler32'], 'meta': i['meta']} for i in files]
         self.did_client.add_files_to_dataset(scope=tmp_scope, name=tmp_dsn, files=files_without_pfn, rse=tmp_rse)
@@ -527,8 +554,10 @@ class TestDIDClients:
         tmp_dsn_output = 'dsn_%s' % generate_uuid()
         self.did_client.create_did_sample(input_scope=tmp_scope, input_name=tmp_dsn, output_scope=tmp_scope, output_name=tmp_dsn_output, nbfiles=2)
         files = [f for f in self.did_client.list_files(scope=tmp_scope, name=tmp_dsn_output)]
-        assert_equal(len(files), 2)
+        assert len(files) == 2
 
+    @pytest.mark.dirty
+    @pytest.mark.noparallel(reason='uses pre-defined RSE')
     def test_attach_dids_to_dids(self):
         """ DATA IDENTIFIERS (CLIENT): Attach dids to dids"""
         tmp_scope = 'mock'
@@ -558,13 +587,15 @@ class TestDIDClients:
         self.did_client.attach_dids_to_dids(attachments=attachments)
         dsns_l = [i for i in self.did_client.get_dataset_by_guid(guid_to_query)]
 
-        assert_equal([dsn], dsns_l)
+        assert [dsn] == dsns_l
 
         cnt_name = 'cnt_%s' % generate_uuid()
         self.did_client.add_container(scope='mock', name=cnt_name)
-        with assert_raises(UnsupportedOperation):
+        with pytest.raises(UnsupportedOperation):
             self.did_client.attach_dids_to_dids([{'scope': 'mock', 'name': cnt_name, 'rse': tmp_rse, 'dids': attachment['dids']}])
 
+    @pytest.mark.dirty
+    @pytest.mark.noparallel(reason='uses pre-defined RSE')
     def test_add_files_to_datasets(self):
         """ DATA IDENTIFIERS (CLIENT): Add files to Datasets"""
         tmp_scope = 'mock'
@@ -592,9 +623,9 @@ class TestDIDClients:
         self.did_client.add_files_to_datasets(attachments)
 
         files = [f for f in self.did_client.list_files(scope=tmp_scope, name=dsn1)]
-        assert_equal(len(files), 10)
+        assert len(files) == 10
 
-        with assert_raises(FileAlreadyExists):
+        with pytest.raises(FileAlreadyExists):
             self.did_client.add_files_to_datasets(attachments)
 
         for attachment in attachments:
@@ -609,7 +640,7 @@ class TestDIDClients:
         self.did_client.add_files_to_datasets(attachments, ignore_duplicate=True)
 
         files = [f for f in self.did_client.list_files(scope=tmp_scope, name=dsn1)]
-        assert_equal(len(files), 15)
+        assert len(files) == 15
 
         # Corrupt meta-data
         files = []
@@ -618,9 +649,10 @@ class TestDIDClients:
                 file['bytes'] = 1000
                 break
 
-        with assert_raises(FileConsistencyMismatch):
+        with pytest.raises(FileConsistencyMismatch):
             self.did_client.add_files_to_datasets(attachments, ignore_duplicate=True)
 
+    @pytest.mark.dirty
     def test_add_dataset(self):
         """ DATA IDENTIFIERS (CLIENT): Add dataset """
         tmp_scope = 'mock'
@@ -630,12 +662,13 @@ class TestDIDClients:
 
         did = self.did_client.get_did(tmp_scope, tmp_dsn)
 
-        assert_equal(did['scope'], tmp_scope)
-        assert_equal(did['name'], tmp_dsn)
+        assert did['scope'] == tmp_scope
+        assert did['name'] == tmp_dsn
 
-        with assert_raises(DataIdentifierNotFound):
+        with pytest.raises(DataIdentifierNotFound):
             self.did_client.get_did('i_dont_exist', 'neither_do_i')
 
+    @pytest.mark.dirty
     def test_add_datasets(self):
         """ DATA IDENTIFIERS (CLIENT): Bulk add datasets """
         tmp_scope = 'mock'
@@ -645,6 +678,8 @@ class TestDIDClients:
             dsns.append(tmp_dsn)
         self.did_client.add_datasets(dsns)
 
+    @pytest.mark.dirty
+    @pytest.mark.noparallel(reason='uses pre-defined RSE')
     def test_exists(self):
         """ DATA IDENTIFIERS (CLIENT): Check if data identifier exists """
         tmp_scope = 'mock'
@@ -655,12 +690,14 @@ class TestDIDClients:
 
         did = self.did_client.get_did(tmp_scope, tmp_file)
 
-        assert_equal(did['scope'], tmp_scope)
-        assert_equal(did['name'], tmp_file)
+        assert did['scope'] == tmp_scope
+        assert did['name'] == tmp_file
 
-        with assert_raises(DataIdentifierNotFound):
+        with pytest.raises(DataIdentifierNotFound):
             self.did_client.get_did('i_dont_exist', 'neither_do_i')
 
+    @pytest.mark.dirty
+    @pytest.mark.noparallel(reason='uses pre-defined RSE')
     def test_did_hierarchy(self):
         """ DATA IDENTIFIERS (CLIENT): Check did hierarchy rule """
 
@@ -692,13 +729,15 @@ class TestDIDClients:
             pass
             # TODO: fix, fix, fix
             # if r['name'] == cnt[1]:
-            #    assert_equal(r['type'], 'container')
-            #    assert_equal(r['level'], 0)
+            #    assert r['type'] == 'container'
+            #    assert r['level'] == 0
             # if (r['name'] == cnt[0]) or (r['name'] == dst[0]) or (r['name'] == file[8]) or (r['name'] == file[9]):
-            #    assert_equal(r['level'], 0)
+            #    assert r['level'] == 0
             # else:
-            #     assert_equal(r['level'], 1)
+            #     assert r['level'] == 1
 
+    @pytest.mark.dirty
+    @pytest.mark.noparallel(reason='uses pre-defined RSE')
     def test_detach_did(self):
         """ DATA IDENTIFIERS (CLIENT): Detach dids from a did"""
 
@@ -724,7 +763,7 @@ class TestDIDClients:
 
         self.did_client.add_containers_to_container(scope, cnt[1], [{'scope': scope, 'name': dst[2]}, {'scope': scope, 'name': dst[3]}])
 
-        with assert_raises(UnsupportedOperation):
+        with pytest.raises(UnsupportedOperation):
             self.did_client.add_datasets_to_container(scope, cnt[0], [{'scope': scope, 'name': dst[1]}, {'scope': scope, 'name': cnt[1]}])
 
         self.did_client.add_datasets_to_container(scope, cnt[0], [{'scope': scope, 'name': dst[1]}, {'scope': scope, 'name': dst[2]}])
@@ -734,14 +773,14 @@ class TestDIDClients:
         result = self.did_client.scope_list(scope, recursive=True)
         for r in result:
             if r['name'] == dst[1]:
-                assert_equal(r['level'], 0)
+                assert r['level'] == 0
             if r['type'] == 'file':
                 if (r['name'] in file[6:9]):
-                    assert_equal(r['level'], 0)
+                    assert r['level'] == 0
                 else:
-                    assert_not_equal(r['level'], 0)
+                    assert r['level'] != 0
 
-        with assert_raises(UnsupportedOperation):
+        with pytest.raises(UnsupportedOperation):
             self.did_client.detach_dids(scope=scope, name=cnt[0], dids=[{'scope': scope, 'name': cnt[0]}])
 
         self.did_client.close(scope, dst[4])
@@ -754,9 +793,11 @@ class TestDIDClients:
         self.did_client.detach_dids(scope, dst[4], [{'scope': scope, 'name': file[8]}, {'scope': scope, 'name': file[9]}])
         metadata = self.did_client.get_metadata(scope, dst[4])
         f_bytes, f_length = metadata['bytes'], metadata['length']
-        assert_equal(i_bytes, f_bytes + file1_bytes + file2_bytes)
-        assert_equal(i_length, f_length + 1 + 1)
+        assert i_bytes == f_bytes + file1_bytes + file2_bytes
+        assert i_length == f_length + 1 + 1
 
+    @pytest.mark.dirty
+    @pytest.mark.noparallel(reason='uses pre-defined RSE')
     def test_scope_list(self):
         """ DATA IDENTIFIERS (CLIENT): Add, aggregate, and list data identifiers in a scope """
 
@@ -801,14 +842,16 @@ class TestDIDClients:
                     r_scope.append(r['scope'])
                 if r['scope'] != self.tmp_scopes[i]:
                     r_otherscopedids.append(r['scope'] + ':' + r['name'])
-                    assert_in(r['level'], [1, 2])
+                    assert r['level'] in [1, 2]
 
             for j in range(3):
-                assert_equal(self.tmp_scopes[i], r_scope[j])
+                assert self.tmp_scopes[i] == r_scope[j]
                 if j != i:
-                    assert_in(self.tmp_scopes[j] + ':' + self.tmp_files[j], r_otherscopedids)
-            assert_not_in(self.tmp_scopes[i] + ':' + self.tmp_files[i], r_topdids)
+                    assert self.tmp_scopes[j] + ':' + self.tmp_files[j] in r_otherscopedids
+            assert self.tmp_scopes[i] + ':' + self.tmp_files[i] not in r_topdids
 
+    @pytest.mark.dirty
+    @pytest.mark.noparallel(reason='uses pre-defined RSE')
     def test_get_did(self):
         """ DATA IDENTIFIERS (CLIENT): add a new data identifier and try to retrieve it back"""
         rse = 'MOCK'
@@ -820,13 +863,15 @@ class TestDIDClients:
 
         did = self.did_client.get_did(scope, file)
 
-        assert_equal(did['scope'], scope)
-        assert_equal(did['name'], file)
+        assert did['scope'] == scope
+        assert did['name'] == file
 
         self.did_client.add_dataset(scope=scope, name=dsn, lifetime=10000000)
         did2 = self.did_client.get_did(scope, dsn)
-        assert_equal(type(did2['expired_at']), datetime)
+        assert type(did2['expired_at']) == datetime
 
+    @pytest.mark.dirty
+    @pytest.mark.noparallel(reason='uses pre-defined RSE')
     def test_get_meta(self):
         """ DATA IDENTIFIERS (CLIENT): add a new meta data for an identifier and try to retrieve it back"""
         rse = 'MOCK'
@@ -842,8 +887,10 @@ class TestDIDClients:
         meta = self.did_client.get_metadata(scope, file)
 
         for i in range(2):
-            assert_equal(meta[keys[i]], values[i])
+            assert meta[keys[i]] == values[i]
 
+    @pytest.mark.dirty
+    @pytest.mark.noparallel(reason='uses pre-defined RSE')
     def test_list_content(self):
         """ DATA IDENTIFIERS (CLIENT): test to list contents for an identifier"""
         rse = 'MOCK'
@@ -857,7 +904,7 @@ class TestDIDClients:
 
         self.did_client.add_dataset(scope, dataset1)
 
-        with assert_raises(DataIdentifierAlreadyExists):
+        with pytest.raises(DataIdentifierAlreadyExists):
             self.did_client.add_dataset(scope, dataset1)
 
         self.did_client.add_files_to_dataset(scope, dataset1, files1, rse=rse)
@@ -872,9 +919,11 @@ class TestDIDClients:
         contents = self.did_client.list_content(scope, container)
 
         datasets_s = [d['name'] for d in contents]
-        assert_in(dataset1, datasets_s)
-        assert_in(dataset2, datasets_s)
+        assert dataset1 in datasets_s
+        assert dataset2 in datasets_s
 
+    @pytest.mark.dirty
+    @pytest.mark.noparallel(reason='uses pre-defined RSE')
     def test_list_files(self):
         """ DATA IDENTIFIERS (CLIENT): List files for a container"""
         rse = 'MOCK'
@@ -903,18 +952,20 @@ class TestDIDClients:
 
         # List file content
         content = self.did_client.list_files(scope, files1[i]['name'])
-        assert_true(content is not None)
+        assert content is not None
         for d in content:
-            assert_true(d['name'] == files1[i]['name'])
+            assert d['name'] == files1[i]['name']
 
         # List container content
         for d in [{'name': x['name'], 'scope': x['scope'], 'bytes': x['bytes'], 'adler32': x['adler32']} for x in self.did_client.list_files(scope, container)]:
-            assert_in(d, files1 + files2)
+            assert d in files1 + files2
 
         # List non-existing data identifier content
-        with assert_raises(DataIdentifierNotFound):
+        with pytest.raises(DataIdentifierNotFound):
             self.did_client.list_files(scope, 'Nimportnawak')
 
+    @pytest.mark.dirty
+    @pytest.mark.noparallel(reason='uses pre-defined RSE')
     def test_list_replicas(self):
         """ DATA IDENTIFIERS (CLIENT): List replicas for a container"""
         rse = 'MOCK'
@@ -938,12 +989,13 @@ class TestDIDClients:
         self.did_client.add_datasets_to_container(scope, cnt, [{'scope': scope, 'name': dsn1}, {'scope': scope, 'name': dsn2}])
 
         replicas = self.replica_client.list_replicas(dids=[{'scope': scope, 'name': dsn1}])
-        assert_true(replicas is not None)
+        assert replicas is not None
 
         replicas = self.replica_client.list_replicas(dids=[{'scope': scope, 'name': cnt}])
-        assert_true(replicas is not None)
+        assert replicas is not None
 
-    @raises(UnsupportedOperation)
+    @pytest.mark.dirty
+    @pytest.mark.noparallel(reason='uses pre-defined RSE')
     def test_close(self):
         """ DATA IDENTIFIERS (CLIENT): test to close data identifiers"""
 
@@ -972,7 +1024,7 @@ class TestDIDClients:
         self.did_client.add_files_to_dataset(scope=tmp_scope, name=tmp_dataset, files=files)
 
         # Close dataset
-        with assert_raises(UnsupportedStatus):
+        with pytest.raises(UnsupportedStatus):
             self.did_client.set_status(scope=tmp_scope, name=tmp_dataset, close=False)
         self.did_client.set_status(scope=tmp_scope, name=tmp_dataset, open=False)
 
@@ -981,9 +1033,11 @@ class TestDIDClients:
         self.replica_client.add_replica(tmp_rse, tmp_scope, tmp_file, 1, '0cc737eb')
         # Add files to dataset
         files = [{'scope': tmp_scope, 'name': tmp_file, 'bytes': 1, 'adler32': '0cc737eb'}, ]
-        self.did_client.attach_dids(scope=tmp_scope, name=tmp_dataset, dids=files)
+        with pytest.raises(exception.UnsupportedOperation):
+            self.did_client.attach_dids(scope=tmp_scope, name=tmp_dataset, dids=files)
 
-    @raises
+    @pytest.mark.dirty
+    @pytest.mark.noparallel(reason='uses pre-defined RSE')
     def test_open(self):
         """ DATA IDENTIFIERS (CLIENT): test to re-open data identifiers for priv account"""
 
@@ -1012,13 +1066,15 @@ class TestDIDClients:
         self.did_client.add_files_to_dataset(scope=tmp_scope, name=tmp_dataset, files=files)
 
         # Close dataset
-        with assert_raises(UnsupportedStatus):
+        with pytest.raises(UnsupportedStatus):
             self.did_client.set_status(scope=tmp_scope, name=tmp_dataset, close=False)
         self.did_client.set_status(scope=tmp_scope, name=tmp_dataset, open=False)
 
         # Add a third file replica
         self.did_client.set_status(scope=tmp_scope, name=tmp_dataset, open=True)
 
+    @pytest.mark.dirty
+    @pytest.mark.noparallel(reason='uses pre-defined RSE')
     def test_bulk_get_meta(self):
         """ DATA IDENTIFIERS (CLIENT): Add a new meta data for a list of DIDs and try to retrieve them back"""
         key = 'project'
@@ -1046,18 +1102,18 @@ class TestDIDClients:
         res_list_dids = [{'scope': entry['scope'], 'name': entry['name']} for entry in list_meta]
         res_list_dids.sort(key=lambda item: item['name'])
         list_dids.sort(key=lambda item: item['name'])
-        assert_equal(list_dids, res_list_dids)
+        assert list_dids == res_list_dids
         for meta in list_meta:
             did = '%s:%s' % (meta['scope'], meta['name'])
             met = meta_mapping[did]
-            assert_equal((key, meta[key]), met)
+            assert (key, meta[key]) == met
         cnt = ['cnt_%s' % generate_uuid() for _ in range(4)]
         for idx in range(4):
             list_dids.append({'scope': scope, 'name': cnt[idx]})
         list_meta = [_ for _ in self.did_client.get_metadata_bulk(list_dids)]
-        assert_equal(len(list_meta), 12)
+        assert len(list_meta) == 12
         list_dids = []
         for idx in range(4):
             list_dids.append({'scope': scope, 'name': cnt[idx]})
         list_meta = [_ for _ in self.did_client.get_metadata_bulk(list_dids)]
-        assert_equal(len(list_meta), 0)
+        assert len(list_meta) == 0
