@@ -1,4 +1,5 @@
-# Copyright 2013-2018 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2014-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,12 +14,20 @@
 # limitations under the License.
 #
 # Authors:
-# - Martin Barisits, <martin.barisits@cern.ch>, 2014-2015
-# - Mario Lassnig, <mario.lassnig@cern.ch>, 2014-2019
-# - Cedric Serfon, <cedric.serfon@cern.ch>, 2019
-# - Andrew Lister, <andrew.lister@stfc.ac.uk>, 2019
-# - Hannes Hansen, <hannes.jakob.hansen@cern.ch>, 2019
+# - Martin Barisits <martin.barisits@cern.ch>, 2014-2016
+# - Vincent Garonne <vincent.garonne@cern.ch>, 2014-2015
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2014-2020
+# - Cedric Serfon <cedric.serfon@cern.ch>, 2019
+# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
+# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2019
+# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
 
+import unittest
+
+import pytest
+
+from rucio.common.config import config_get, config_get_bool
 from rucio.common.types import InternalAccount, InternalScope
 from rucio.common.utils import generate_uuid as uuid
 from rucio.core.account import get_usage
@@ -27,26 +36,32 @@ from rucio.core.did import add_did, attach_dids, detach_dids
 from rucio.core.lock import get_replica_locks, get_dataset_locks
 from rucio.core.rse import add_rse_attribute, get_rse_id
 from rucio.core.rule import add_rule, get_rule
-from rucio.daemons.judge.evaluator import re_evaluator
 from rucio.daemons.abacus.account import account_update
+from rucio.daemons.judge.evaluator import re_evaluator
 from rucio.db.sqla.constants import DIDType
 from rucio.tests.test_rule import create_files, tag_generator
 
 
-class TestJudgeEvaluator():
+@pytest.mark.noparallel(reason='sets account limits, adds global rse attributes')
+class TestJudgeEvaluator(unittest.TestCase):
 
     @classmethod
     def setUpClass(cls):
+        if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
+            cls.vo = {'vo': config_get('client', 'vo', raise_exception=False, default='tst')}
+        else:
+            cls.vo = {}
+
         # Add test RSE
         cls.rse1 = 'MOCK'
         cls.rse3 = 'MOCK3'
         cls.rse4 = 'MOCK4'
         cls.rse5 = 'MOCK5'
 
-        cls.rse1_id = get_rse_id(rse=cls.rse1)
-        cls.rse3_id = get_rse_id(rse=cls.rse3)
-        cls.rse4_id = get_rse_id(rse=cls.rse4)
-        cls.rse5_id = get_rse_id(rse=cls.rse5)
+        cls.rse1_id = get_rse_id(rse=cls.rse1, **cls.vo)
+        cls.rse3_id = get_rse_id(rse=cls.rse3, **cls.vo)
+        cls.rse4_id = get_rse_id(rse=cls.rse4, **cls.vo)
+        cls.rse5_id = get_rse_id(rse=cls.rse5, **cls.vo)
 
         # Add Tags
         cls.T1 = tag_generator()
@@ -63,8 +78,8 @@ class TestJudgeEvaluator():
         add_rse_attribute(cls.rse5_id, "fakeweight", 0)
 
         # Add quota
-        cls.jdoe = InternalAccount('jdoe')
-        cls.root = InternalAccount('root')
+        cls.jdoe = InternalAccount('jdoe', **cls.vo)
+        cls.root = InternalAccount('root', **cls.vo)
         set_local_account_limit(cls.jdoe, cls.rse1_id, -1)
         set_local_account_limit(cls.jdoe, cls.rse3_id, -1)
         set_local_account_limit(cls.jdoe, cls.rse4_id, -1)
@@ -77,10 +92,10 @@ class TestJudgeEvaluator():
 
     def test_judge_add_files_to_dataset(self):
         """ JUDGE EVALUATOR: Test the judge when adding files to dataset"""
-        scope = InternalScope('mock')
+        scope = InternalScope('mock', **self.vo)
         files = create_files(3, scope, self.rse1_id)
         dataset = 'dataset_' + str(uuid())
-        add_did(scope, dataset, DIDType.from_sym('DATASET'), self.jdoe)
+        add_did(scope, dataset, DIDType.DATASET, self.jdoe)
 
         # Add a first rule to the DS
         add_rule(dids=[{'scope': scope, 'name': dataset}], account=self.jdoe, copies=2, rse_expression=self.T1, grouping='DATASET', weight=None, lifetime=None, locked=False, subscription_id=None)
@@ -100,14 +115,14 @@ class TestJudgeEvaluator():
 
     def test_judge_add_dataset_to_container(self):
         """ JUDGE EVALUATOR: Test the judge when adding dataset to container"""
-        scope = InternalScope('mock')
+        scope = InternalScope('mock', **self.vo)
         files = create_files(3, scope, self.rse1_id)
         dataset = 'dataset_' + str(uuid())
-        add_did(scope, dataset, DIDType.from_sym('DATASET'), self.jdoe)
+        add_did(scope, dataset, DIDType.DATASET, self.jdoe)
         attach_dids(scope, dataset, files, self.jdoe)
 
         parent_container = 'dataset_' + str(uuid())
-        add_did(scope, parent_container, DIDType.from_sym('CONTAINER'), self.jdoe)
+        add_did(scope, parent_container, DIDType.CONTAINER, self.jdoe)
         # Add a first rule to the DS
         add_rule(dids=[{'scope': scope, 'name': parent_container}], account=self.jdoe, copies=2, rse_expression=self.T1, grouping='DATASET', weight=None, lifetime=None, locked=False, subscription_id=None)
         attach_dids(scope, parent_container, [{'scope': scope, 'name': dataset}], self.jdoe)
@@ -127,10 +142,10 @@ class TestJudgeEvaluator():
         re_evaluator(once=True)
         account_update(once=True)
 
-        scope = InternalScope('mock')
+        scope = InternalScope('mock', **self.vo)
         files = create_files(3, scope, self.rse1_id, bytes=100)
         dataset = 'dataset_' + str(uuid())
-        add_did(scope, dataset, DIDType.from_sym('DATASET'), self.jdoe)
+        add_did(scope, dataset, DIDType.DATASET, self.jdoe)
 
         # Add a first rule to the DS
         add_rule(dids=[{'scope': scope, 'name': dataset}], account=self.jdoe, copies=1, rse_expression=self.rse1, grouping='ALL', weight=None, lifetime=None, locked=False, subscription_id=None)
@@ -151,10 +166,10 @@ class TestJudgeEvaluator():
         re_evaluator(once=True)
         account_update(once=True)
 
-        scope = InternalScope('mock')
+        scope = InternalScope('mock', **self.vo)
         files = create_files(3, scope, self.rse1_id, bytes=100)
         dataset = 'dataset_' + str(uuid())
-        add_did(scope, dataset, DIDType.from_sym('DATASET'), self.jdoe)
+        add_did(scope, dataset, DIDType.DATASET, self.jdoe)
         attach_dids(scope, dataset, files, self.jdoe)
 
         # Add a first rule to the DS
@@ -178,14 +193,14 @@ class TestJudgeEvaluator():
         """ JUDGE EVALUATOR: Test if the a datasetlock is detached correctly when removing a dataset from a container"""
         re_evaluator(once=True)
 
-        scope = InternalScope('mock')
+        scope = InternalScope('mock', **self.vo)
         files = create_files(3, scope, self.rse1_id, bytes=100)
         dataset = 'dataset_' + str(uuid())
-        add_did(scope, dataset, DIDType.from_sym('DATASET'), self.jdoe)
+        add_did(scope, dataset, DIDType.DATASET, self.jdoe)
         attach_dids(scope, dataset, files, self.jdoe)
 
         container = 'container_' + str(uuid())
-        add_did(scope, container, DIDType.from_sym('CONTAINER'), self.jdoe)
+        add_did(scope, container, DIDType.CONTAINER, self.jdoe)
         attach_dids(scope, container, [{'scope': scope, 'name': dataset}], self.jdoe)
 
         # Add a rule to the Container
@@ -207,28 +222,28 @@ class TestJudgeEvaluator():
         """ JUDGE EVALUATOR: Test if the detach is done correctly"""
         re_evaluator(once=True)
 
-        scope = InternalScope('mock')
+        scope = InternalScope('mock', **self.vo)
         container = 'container_' + str(uuid())
-        add_did(scope, container, DIDType.from_sym('CONTAINER'), self.jdoe)
+        add_did(scope, container, DIDType.CONTAINER, self.jdoe)
 
-        scope = InternalScope('mock')
+        scope = InternalScope('mock', **self.vo)
         files = create_files(3, scope, self.rse1_id, bytes=100)
         dataset = 'dataset_' + str(uuid())
-        add_did(scope, dataset, DIDType.from_sym('DATASET'), self.jdoe)
+        add_did(scope, dataset, DIDType.DATASET, self.jdoe)
         attach_dids(scope, dataset, files, self.jdoe)
         attach_dids(scope, container, [{'scope': scope, 'name': dataset}], self.jdoe)
 
-        scope = InternalScope('mock')
+        scope = InternalScope('mock', **self.vo)
         files = create_files(3, scope, self.rse1_id, bytes=100)
         dataset = 'dataset_' + str(uuid())
-        add_did(scope, dataset, DIDType.from_sym('DATASET'), self.jdoe)
+        add_did(scope, dataset, DIDType.DATASET, self.jdoe)
         attach_dids(scope, dataset, files, self.jdoe)
         attach_dids(scope, container, [{'scope': scope, 'name': dataset}], self.jdoe)
 
-        scope = InternalScope('mock')
+        scope = InternalScope('mock', **self.vo)
         files = create_files(3, scope, self.rse1_id, bytes=100)
         dataset = 'dataset_' + str(uuid())
-        add_did(scope, dataset, DIDType.from_sym('DATASET'), self.jdoe)
+        add_did(scope, dataset, DIDType.DATASET, self.jdoe)
         attach_dids(scope, dataset, files, self.jdoe)
         attach_dids(scope, container, [{'scope': scope, 'name': dataset}], self.jdoe)
 
@@ -249,10 +264,10 @@ class TestJudgeEvaluator():
 
     def test_judge_add_files_to_dataset_with_2_rules(self):
         """ JUDGE EVALUATOR: Test the judge when adding files to dataset with 2 rules"""
-        scope = InternalScope('mock')
+        scope = InternalScope('mock', **self.vo)
         files = create_files(3, scope, self.rse1_id)
         dataset = 'dataset_' + str(uuid())
-        add_did(scope, dataset, DIDType.from_sym('DATASET'), self.jdoe)
+        add_did(scope, dataset, DIDType.DATASET, self.jdoe)
 
         # Add a first rule to the DS
         add_rule(dids=[{'scope': scope, 'name': dataset}], account=self.jdoe, copies=1, rse_expression=self.rse5, grouping='DATASET', weight=None, lifetime=None, locked=False, subscription_id=None)
@@ -273,18 +288,18 @@ class TestJudgeEvaluator():
 
     def test_judge_add_files_to_dataset_rule_on_container(self):
         """ JUDGE EVALUATOR: Test the judge when attaching file to dataset with rule on two levels of containers"""
-        scope = InternalScope('mock')
+        scope = InternalScope('mock', **self.vo)
         files = create_files(3, scope, self.rse1_id)
         dataset = 'dataset_' + str(uuid())
-        add_did(scope, dataset, DIDType.from_sym('DATASET'), self.jdoe)
+        add_did(scope, dataset, DIDType.DATASET, self.jdoe)
         attach_dids(scope, dataset, files, self.jdoe)
 
         parent_container = 'dataset_' + str(uuid())
-        add_did(scope, parent_container, DIDType.from_sym('CONTAINER'), self.jdoe)
+        add_did(scope, parent_container, DIDType.CONTAINER, self.jdoe)
         attach_dids(scope, parent_container, [{'scope': scope, 'name': dataset}], self.jdoe)
 
         parent_parent_container = 'dataset_' + str(uuid())
-        add_did(scope, parent_parent_container, DIDType.from_sym('CONTAINER'), self.jdoe)
+        add_did(scope, parent_parent_container, DIDType.CONTAINER, self.jdoe)
         attach_dids(scope, parent_parent_container, [{'scope': scope, 'name': parent_container}], self.jdoe)
 
         # Add a first rule to the DS
