@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 CERN
+# Copyright 2020-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,19 +16,20 @@
 # Authors:
 # - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
 # - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2020
-# - Martin Barisits <martin.barisits@cern.ch>, 2020
+# - Martin Barisits <martin.barisits@cern.ch>, 2020-2021
+# - Radu Carpa <radu.carpa@cern.ch>, 2021
 
 import os
-import sys
 import unittest
-from datetime import datetime
 from logging import getLogger
 from os import remove
 from random import choice
 from re import search
 from string import ascii_uppercase, ascii_lowercase
+from unittest.mock import patch
+from urllib.parse import urlparse, parse_qs
 
 import pytest
 from oic import rndstr
@@ -41,7 +42,7 @@ from rucio.api.did import add_did, list_dids
 from rucio.api.identity import add_account_identity, list_accounts_for_identity
 from rucio.api.lock import get_replica_locks_for_rule_id
 from rucio.api.replica import list_replicas
-from rucio.api.rse import add_protocol, add_rse, add_rse_attribute, list_rses, set_rse_limits, set_rse_usage
+from rucio.api.rse import add_protocol, add_rse, add_rse_attribute, list_rses
 from rucio.api.rule import delete_replication_rule, get_replication_rule
 from rucio.api.scope import add_scope, list_scopes
 from rucio.api.subscription import add_subscription, list_subscriptions
@@ -65,24 +66,11 @@ from rucio.core.rse_expression_parser import parse_expression
 from rucio.core.rule import add_rule
 from rucio.core.vo import add_vo, vo_exists
 from rucio.daemons.automatix.automatix import automatix
-from rucio.daemons.reaper.reaper import run as run_reaper
 from rucio.db.sqla import models, session as db_session
 from rucio.tests.common import execute, headers, hdrdict, vohdr, auth, loginhdr
 from rucio.tests.test_authentication import PRIVATE_KEY, PUBLIC_KEY
 from rucio.tests.test_oidc import get_mock_oidc_client, NEW_TOKEN_DICT
 
-try:
-    # Python 2
-    from urlparse import urlparse, parse_qs
-except ImportError:
-    # Python 3
-    from urllib.parse import urlparse, parse_qs
-
-if sys.version_info >= (3, 3):
-    from unittest.mock import patch
-else:
-    from mock import patch
-
 LOG = getLogger(__name__)
 
 # module-level skip, see https://docs.pytest.org/en/latest/skipping.html#skip-all-test-functions-of-a-class-or-module
@@ -99,7 +87,7 @@ def setup_vo():
             add_vo(description='Test', email='rucio@email.com', **new_vo)
         return vo, new_vo
     else:
-        pytest.xfail(reason='multi_vo mode is not enabled. Running multi_vo tests in single_vo mode would result in failures.')
+        pytest.skip('multi_vo mode is not enabled. Running multi_vo tests in single_vo mode would result in failures.')
         return {}, {}
 
 
@@ -109,6 +97,7 @@ class TestVOCoreAPI(unittest.TestCase):
     def setUpClass(cls):
         cls.vo, cls.new_vo = setup_vo()
 
+    @pytest.mark.noparallel(reason='changes global configuration value')
     def test_multi_vo_flag(self):
         """ MULTI VO (CORE): Test operations fail in single_vo mode """
         try:
@@ -125,6 +114,7 @@ class TestVOCoreAPI(unittest.TestCase):
             else:
                 config_remove_option('common', 'multi_vo')
 
+    @pytest.mark.noparallel(reason='uses global RSE (MOCK) and fails when run in parallel')
     def test_access_rule_vo(self):
         """ MULTI VO (CORE): Test accessing rules from a different VO """
         scope = InternalScope('mock', **self.vo)
@@ -196,6 +186,7 @@ class TestVOCoreAPI(unittest.TestCase):
         add_scope(scope, 'super_root', 'super_root', vo='def')
         assert scope in [s for s in list_scopes(filter={}, vo='def')]
 
+    @pytest.mark.noparallel(reason='changes global configuration value')
     def test_super_root_naming(self):
         """ MULTI VO (CORE): Test we can only name accounts super_root when appropriate """
         with pytest.raises(Duplicate):  # Ensure we fail from duplication rather than the choice of name
@@ -512,6 +503,7 @@ class TestVORestAPI(unittest.TestCase):
         response = self.rest_client.get('/vos/', headers=headers(auth(token)))
         assert response.status_code == 401
 
+    @pytest.mark.noparallel(reason='changes global configuration value')
     def test_list_vos_unsupported(self):
         """ MULTI VO (REST): Test list VOs through REST layer raises UnsupportedOperation """
         response = self.rest_client.get('/auth/userpass', headers=headers(loginhdr('super_root', 'ddmlab', 'secret'), vohdr('def')))
@@ -546,6 +538,7 @@ class TestVORestAPI(unittest.TestCase):
         response = self.rest_client.post('/vos/' + self.vo['vo'], headers=headers(auth(token)), json=params)
         assert response.status_code == 401
 
+    @pytest.mark.noparallel(reason='changes global configuration value')
     def test_add_vo_unsupported(self):
         """ MULTI VO (REST): Test adding VO through REST layer raises UnsupportedOperation """
         response = self.rest_client.get('/auth/userpass', headers=headers(loginhdr('super_root', 'ddmlab', 'secret'), vohdr('def')))
@@ -610,6 +603,7 @@ class TestVORestAPI(unittest.TestCase):
         response = self.rest_client.put('/vos/' + self.vo['vo'], headers=headers(auth(token)), json=params)
         assert response.status_code == 401
 
+    @pytest.mark.noparallel(reason='changes global configuration value')
     def test_update_vo_unsupported(self):
         """ MULTI VO (REST): Test updating VO through REST layer raises UnsupportedOperation """
         response = self.rest_client.get('/auth/userpass', headers=headers(loginhdr('super_root', 'ddmlab', 'secret'), vohdr('def')))
@@ -1040,6 +1034,7 @@ class TestMultiVOBinRucio(unittest.TestCase):
         assert self.rse_new not in out
 
 
+@pytest.mark.noparallel(reason='runs daemons, fails when run in parallel')
 class TestMultiVODaemons(unittest.TestCase):
 
     @classmethod
@@ -1088,66 +1083,3 @@ class TestMultiVODaemons(unittest.TestCase):
         replicas_new = list(list_replicas(did_dicts, rse_expression=shr_rse, **self.new_vo))
         assert len(replicas_tst) != 0
         assert len(replicas_new) == 0
-
-    def test_reaper(self):
-        """ MULTI VO (DAEMON): Test that reaper runs on the specified VO(s) """
-        rse_str = ''.join(choice(ascii_uppercase) for x in range(10))
-        rse_name = 'SHR_%s' % rse_str
-        rse_id_tst = add_rse(rse_name, 'root', **self.vo)
-        rse_id_new = add_rse(rse_name, 'root', **self.new_vo)
-
-        mock_protocol = {'scheme': 'MOCK',
-                         'hostname': 'localhost',
-                         'port': 123,
-                         'prefix': '/test/reaper',
-                         'impl': 'rucio.rse.protocols.mock.Default',
-                         'domains': {
-                             'lan': {'read': 1,
-                                     'write': 1,
-                                     'delete': 1},
-                             'wan': {'read': 1,
-                                     'write': 1,
-                                     'delete': 1}}}
-        add_protocol(rse=rse_name, data=mock_protocol, issuer='root', **self.vo)
-        add_protocol(rse=rse_name, data=mock_protocol, issuer='root', **self.new_vo)
-
-        scope_uuid = str(generate_uuid()).lower()[:16]
-        scope_name = 'shr_%s' % scope_uuid
-        scope_tst = InternalScope(scope_name, **self.vo)
-        scope_new = InternalScope(scope_name, **self.new_vo)
-        add_scope(scope_name, 'root', 'root', **self.vo)
-        add_scope(scope_name, 'root', 'root', **self.new_vo)
-
-        nb_files = 30
-        file_size = 2147483648  # 2G
-
-        names = []
-        for i in range(nb_files):
-            name = 'lfn%s' % generate_uuid()
-            names.append(name)
-            add_replica(rse_id=rse_id_tst, scope=scope_tst, name=name, bytes=file_size, account=InternalAccount('root', **self.vo),
-                        adler32=None, md5=None, tombstone=datetime.utcnow())
-            add_replica(rse_id=rse_id_new, scope=scope_new, name=name, bytes=file_size, account=InternalAccount('root', **self.new_vo),
-                        adler32=None, md5=None, tombstone=datetime.utcnow())
-
-        set_rse_usage(rse=rse_name, source='storage', used=nb_files * file_size, free=800, issuer='root', **self.vo)
-        set_rse_limits(rse=rse_name, name='MinFreeSpace', value=10737418240, issuer='root', **self.vo)
-        set_rse_limits(rse=rse_name, name='MaxBeingDeletedFiles', value=10, issuer='root', **self.vo)
-
-        set_rse_usage(rse=rse_name, source='storage', used=nb_files * file_size, free=800, issuer='root', **self.new_vo)
-        set_rse_limits(rse=rse_name, name='MinFreeSpace', value=10737418240, issuer='root', **self.new_vo)
-        set_rse_limits(rse=rse_name, name='MaxBeingDeletedFiles', value=10, issuer='root', **self.new_vo)
-
-        # Check we start of with the expected number of replicas
-        assert len(list(list_replicas([{'scope': scope_name, 'name': n} for n in names], rse_expression=rse_name, **self.vo))) == nb_files
-        assert len(list(list_replicas([{'scope': scope_name, 'name': n} for n in names], rse_expression=rse_name, **self.new_vo))) == nb_files
-
-        # Check we reap all VOs by default
-        run_reaper(once=True, rses=[rse_name])
-        assert len(list(list_replicas([{'scope': scope_name, 'name': n} for n in names], rse_expression=rse_name, **self.vo))) == nb_files - 5
-        assert len(list(list_replicas([{'scope': scope_name, 'name': n} for n in names], rse_expression=rse_name, **self.new_vo))) == nb_files - 5
-
-        # Check we don't affect a second VO that isn't specified
-        run_reaper(once=True, rses=[rse_name], vos=['new'])
-        assert len(list(list_replicas([{'scope': scope_name, 'name': n} for n in names], rse_expression=rse_name, **self.vo))) == nb_files - 5
-        assert len(list(list_replicas([{'scope': scope_name, 'name': n} for n in names], rse_expression=rse_name, **self.new_vo))) == nb_files - 10
