@@ -23,10 +23,11 @@
 import unittest
 from datetime import datetime, timedelta
 
+import pytest
+
 from rucio.common.config import config_get, config_get_bool, config_set, config_remove_option
 from rucio.common.types import InternalAccount, InternalScope
 from rucio.common.utils import generate_uuid
-from rucio.core.config import set
 from rucio.core.did import attach_dids, add_did
 from rucio.core.replica import add_replica
 from rucio.core.request import queue_requests, get_request_by_did, release_waiting_requests_per_deadline, release_all_waiting_requests, release_waiting_requests_fifo, release_waiting_requests_grouped_fifo, release_waiting_requests_per_free_volume
@@ -37,6 +38,12 @@ from rucio.db.sqla.constants import DIDType, RequestType, RequestState
 from rucio.tests.common import skiplimitedsql
 
 
+@pytest.mark.dirty
+@pytest.mark.noparallel(reason='deletes database content on setUp, uses pre-defined rses, changes global configuration value')
+@pytest.mark.usefixtures("core_config_mock")
+@pytest.mark.parametrize("core_config_mock", [{
+    "table_content": [('throttler', 'mode', 'DEST_PER_ALL_ACT')]
+}], indirect=True)
 class TestThrottlerGroupedFIFO(unittest.TestCase):
     """Throttler per destination RSE and on all activites per grouped FIFO
     """
@@ -50,8 +57,6 @@ class TestThrottlerGroupedFIFO(unittest.TestCase):
         else:
             cls.vo = {}
 
-        cls.db_session = session.get_session()
-        cls.dialect = cls.db_session.bind.dialect.name
         cls.dest_rse = 'MOCK'
         cls.source_rse = 'MOCK4'
         cls.dest_rse_id = get_rse_id(cls.dest_rse, **cls.vo)
@@ -63,17 +68,19 @@ class TestThrottlerGroupedFIFO(unittest.TestCase):
         config_set('conveyor', 'use_preparer', 'true')
 
     def setUp(self):
+        self.db_session = session.get_session()
+        self.dialect = self.db_session.bind.dialect.name
         self.db_session.query(models.Request).delete()
         self.db_session.query(models.RSETransferLimit).delete()
-        self.db_session.query(models.Config).delete()
-        set('throttler', 'mode', 'DEST_PER_ALL_ACT', session=self.db_session)
         self.db_session.commit()
 
+    def tearDown(self):
+        self.db_session.commit()
+        self.db_session.close()
+
     @classmethod
     def tearDownClass(cls):
         config_remove_option('conveyor', 'use_preparer')
-        cls.db_session.commit()
-        cls.db_session.close()
 
     def test_preparer_throttler_grouped_fifo_all(self):
         """ THROTTLER (CLIENTS): throttler release all waiting requests (DEST - ALL ACT - GFIFO). """
@@ -405,8 +412,15 @@ class TestThrottlerGroupedFIFO(unittest.TestCase):
         # deadline check should not work for destination RSEs - only for reading
 
 
-# Throttler per destination RSE and on each activites per FIFO
+@pytest.mark.dirty
+@pytest.mark.noparallel(reason='deletes database content on setUp, uses pre-defined rses, changes global configuration value')
+@pytest.mark.usefixtures("core_config_mock")
+@pytest.mark.parametrize("core_config_mock", [{
+    "table_content": [('throttler', 'mode', 'DEST_PER_ACT')]
+}], indirect=True)
 class TestThrottlerFIFO(unittest.TestCase):
+    """Throttler per destination RSE and on each activites per FIFO
+    """
     db_session = None
 
     @classmethod
@@ -416,8 +430,6 @@ class TestThrottlerFIFO(unittest.TestCase):
         else:
             cls.vo = {}
 
-        cls.db_session = session.get_session()
-        cls.dialect = cls.db_session.bind.dialect.name
         cls.dest_rse = 'MOCK'
         cls.source_rse = 'MOCK4'
         cls.dest_rse_id = get_rse_id(cls.dest_rse, **cls.vo)
@@ -429,17 +441,19 @@ class TestThrottlerFIFO(unittest.TestCase):
         config_set('conveyor', 'use_preparer', 'true')
 
     def setUp(self):
+        self.db_session = session.get_session()
+        self.dialect = self.db_session.bind.dialect.name
         self.db_session.query(models.Request).delete()
         self.db_session.query(models.RSETransferLimit).delete()
-        self.db_session.query(models.Config).delete()
-        set('throttler', 'mode', 'DEST_PER_ACT', session=self.db_session)
         self.db_session.commit()
 
+    def tearDown(self):
+        self.db_session.commit()
+        self.db_session.close()
+
     @classmethod
     def tearDownClass(cls):
         config_remove_option('conveyor', 'use_preparer')
-        cls.db_session.commit()
-        cls.db_session.close()
 
     def test_throttler_fifo_release_all(self):
         """ THROTTLER (CLIENTS): throttler release all waiting requests (DEST - ACT - FIFO). """
@@ -651,6 +665,12 @@ class TestThrottlerFIFO(unittest.TestCase):
         assert request2['state'] == RequestState.WAITING
 
 
+@pytest.mark.dirty
+@pytest.mark.noparallel(reason='deletes database content on setUp, uses pre-defined rses, changes global configuration value')
+@pytest.mark.usefixtures("core_config_mock")
+@pytest.mark.parametrize("core_config_mock", [{
+    "table_content": [('throttler', 'mode', 'SRC_PER_ACT')]
+}], indirect=True)
 class TestThrottlerFIFOSRCACT(unittest.TestCase):
     """Throttler per source RSE and on each activites per FIFO."""
 
@@ -663,8 +683,6 @@ class TestThrottlerFIFOSRCACT(unittest.TestCase):
         else:
             cls.vo = {}
 
-        cls.db_session = session.get_session()
-        cls.dialect = cls.db_session.bind.dialect.name
         cls.dest_rse = 'MOCK'
         cls.dest_rse2 = 'MOCK5'
         cls.dest_rse3 = 'MOCK3'
@@ -681,17 +699,19 @@ class TestThrottlerFIFOSRCACT(unittest.TestCase):
         config_set('conveyor', 'use_preparer', 'true')
 
     def setUp(self):
+        self.db_session = session.get_session()
+        self.dialect = self.db_session.bind.dialect.name
         self.db_session.query(models.Request).delete()
         self.db_session.query(models.RSETransferLimit).delete()
-        self.db_session.query(models.Config).delete()
-        set('throttler', 'mode', 'SRC_PER_ACT', session=self.db_session)
         self.db_session.commit()
 
+    def tearDown(self):
+        self.db_session.commit()
+        self.db_session.close()
+
     @classmethod
     def tearDownClass(cls):
         config_remove_option('conveyor', 'use_preparer')
-        cls.db_session.commit()
-        cls.db_session.close()
 
     def test_throttler_fifo_release_subset(self):
         """ THROTTLER (CLIENTS): throttler release subset of waiting requests (SRC - ACT - FIFO). """
@@ -774,6 +794,12 @@ class TestThrottlerFIFOSRCACT(unittest.TestCase):
         assert request3['state'] == RequestState.WAITING
 
 
+@pytest.mark.dirty
+@pytest.mark.noparallel(reason='deletes database content on setUp, uses pre-defined rses, changes global configuration value')
+@pytest.mark.usefixtures("core_config_mock")
+@pytest.mark.parametrize("core_config_mock", [{
+    "table_content": [('throttler', 'mode', 'SRC_PER_ALL_ACT')]
+}], indirect=True)
 class TestThrottlerFIFOSRCALLACT(unittest.TestCase):
     """Throttler per source RSE and on all activites per FIFO."""
 
@@ -786,8 +812,6 @@ class TestThrottlerFIFOSRCALLACT(unittest.TestCase):
         else:
             cls.vo = {}
 
-        cls.db_session = session.get_session()
-        cls.dialect = cls.db_session.bind.dialect.name
         cls.dest_rse = 'MOCK'
         cls.source_rse = 'MOCK4'
         cls.dest_rse_id = get_rse_id(cls.dest_rse, **cls.vo)
@@ -799,17 +823,19 @@ class TestThrottlerFIFOSRCALLACT(unittest.TestCase):
         config_set('conveyor', 'use_preparer', 'true')
 
     def setUp(self):
+        self.db_session = session.get_session()
+        self.dialect = self.db_session.bind.dialect.name
         self.db_session.query(models.Request).delete()
         self.db_session.query(models.RSETransferLimit).delete()
-        self.db_session.query(models.Config).delete()
-        set('throttler', 'mode', 'SRC_PER_ALL_ACT', session=self.db_session)
         self.db_session.commit()
 
+    def tearDown(self):
+        self.db_session.commit()
+        self.db_session.close()
+
     @classmethod
     def tearDownClass(cls):
         config_remove_option('conveyor', 'use_preparer')
-        cls.db_session.commit()
-        cls.db_session.close()
 
     def test_throttler_fifo_release_subset(self):
         """ THROTTLER (CLIENTS): throttler release subset of waiting requests (SRC - ALL ACT - FIFO). """
@@ -869,6 +895,12 @@ class TestThrottlerFIFOSRCALLACT(unittest.TestCase):
         assert request2['state'] == RequestState.WAITING
 
 
+@pytest.mark.dirty
+@pytest.mark.noparallel(reason='deletes database content on setUp, uses pre-defined rses, changes global configuration value')
+@pytest.mark.usefixtures("core_config_mock")
+@pytest.mark.parametrize("core_config_mock", [{
+    "table_content": [('throttler', 'mode', 'DEST_PER_ALL_ACT')]
+}], indirect=True)
 class TestThrottlerFIFODESTALLACT(unittest.TestCase):
     """Throttler per destination RSE and on all activites per FIFO."""
 
@@ -881,8 +913,6 @@ class TestThrottlerFIFODESTALLACT(unittest.TestCase):
         else:
             cls.vo = {}
 
-        cls.db_session = session.get_session()
-        cls.dialect = cls.db_session.bind.dialect.name
         cls.dest_rse = 'MOCK'
         cls.dest_rse2 = 'MOCK5'
         cls.source_rse = 'MOCK4'
@@ -899,17 +929,19 @@ class TestThrottlerFIFODESTALLACT(unittest.TestCase):
         config_set('conveyor', 'use_preparer', 'true')
 
     def setUp(self):
+        self.db_session = session.get_session()
+        self.dialect = self.db_session.bind.dialect.name
         self.db_session.query(models.Request).delete()
         self.db_session.query(models.RSETransferLimit).delete()
-        self.db_session.query(models.Config).delete()
-        set('throttler', 'mode', 'DEST_PER_ALL_ACT', session=self.db_session)
         self.db_session.commit()
 
+    def tearDown(self):
+        self.db_session.commit()
+        self.db_session.close()
+
     @classmethod
     def tearDownClass(cls):
         config_remove_option('conveyor', 'use_preparer')
-        cls.db_session.commit()
-        cls.db_session.close()
 
     def test_throttler_fifo_release_subset(self):
         """ THROTTLER (CLIENTS): throttler release subset of waiting requests (DEST - ALL ACT - FIFO). """
@@ -994,6 +1026,12 @@ class TestThrottlerFIFODESTALLACT(unittest.TestCase):
         assert request3['state'] == RequestState.WAITING
 
 
+@pytest.mark.dirty
+@pytest.mark.noparallel(reason='deletes database content on setUp, uses pre-defined rses, changes global configuration value')
+@pytest.mark.usefixtures("core_config_mock")
+@pytest.mark.parametrize("core_config_mock", [{
+    "table_content": [('throttler', 'mode', 'SRC_PER_ALL_ACT')]
+}], indirect=True)
 class TestThrottlerGroupedFIFOSRCALLACT(unittest.TestCase):
     """Throttler per source RSE and on all activites per grouped FIFO."""
 
@@ -1006,8 +1044,6 @@ class TestThrottlerGroupedFIFOSRCALLACT(unittest.TestCase):
         else:
             cls.vo = {}
 
-        cls.db_session = session.get_session()
-        cls.dialect = cls.db_session.bind.dialect.name
         cls.dest_rse = 'MOCK'
         cls.source_rse = 'MOCK4'
         cls.dest_rse_2 = 'MOCK3'
@@ -1021,17 +1057,19 @@ class TestThrottlerGroupedFIFOSRCALLACT(unittest.TestCase):
         config_set('conveyor', 'use_preparer', 'true')
 
     def setUp(self):
+        self.db_session = session.get_session()
+        self.dialect = self.db_session.bind.dialect.name
         self.db_session.query(models.Request).delete()
         self.db_session.query(models.RSETransferLimit).delete()
-        self.db_session.query(models.Config).delete()
-        set('throttler', 'mode', 'SRC_PER_ALL_ACT', session=self.db_session)
         self.db_session.commit()
 
+    def tearDown(self):
+        self.db_session.commit()
+        self.db_session.close()
+
     @classmethod
     def tearDownClass(cls):
         config_remove_option('conveyor', 'use_preparer')
-        cls.db_session.commit()
-        cls.db_session.close()
 
     @skiplimitedsql
     def test_preparer_throttler_grouped_fifo_subset(self):
@@ -1151,6 +1189,12 @@ class TestThrottlerGroupedFIFOSRCALLACT(unittest.TestCase):
         assert request_4['state'] == RequestState.WAITING
 
 
+@pytest.mark.dirty
+@pytest.mark.noparallel(reason='deletes database content on setUp, uses pre-defined rses, changes global configuration value')
+@pytest.mark.usefixtures("core_config_mock")
+@pytest.mark.parametrize("core_config_mock", [{
+    "table_content": [('throttler', 'mode', 'DEST_PER_ACT')]
+}], indirect=True)
 class TestRequestCoreRelease(unittest.TestCase):
     """Test release methods used in throttler."""
 
@@ -1163,8 +1207,6 @@ class TestRequestCoreRelease(unittest.TestCase):
         else:
             cls.vo = {}
 
-        cls.db_session = session.get_session()
-        cls.dialect = cls.db_session.bind.dialect.name
         cls.dest_rse = 'MOCK'
         cls.source_rse = 'MOCK4'
         cls.source_rse2 = 'MOCK5'
@@ -1178,15 +1220,15 @@ class TestRequestCoreRelease(unittest.TestCase):
         config_set('conveyor', 'use_preparer', 'true')
 
     def setUp(self):
+        self.db_session = session.get_session()
+        self.dialect = self.db_session.bind.dialect.name
         self.db_session.query(models.Request).delete()
         self.db_session.query(models.RSETransferLimit).delete()
         self.db_session.query(models.Distance).delete()
-        self.db_session.query(models.Config).delete()
         # set transfer limits to put requests in waiting state
         set_rse_transfer_limits(self.dest_rse_id, self.user_activity, max_transfers=1, session=self.db_session)
         set_rse_transfer_limits(self.dest_rse_id, self.all_activities, max_transfers=1, session=self.db_session)
         set_rse_transfer_limits(self.dest_rse_id, 'ignore', max_transfers=1, session=self.db_session)
-        set('throttler', 'mode', 'DEST_PER_ACT', session=self.db_session)
         self.db_session.commit()
 
     def tearDown(self):
@@ -1378,7 +1420,7 @@ class TestRequestCoreRelease(unittest.TestCase):
         self.db_session.query(models.Request).delete()
         self.db_session.commit()
         name1 = generate_uuid()
-        add_replica(self.dest_rse_id, self.scope, name1, 1, self.account, session=self.db_session)
+        add_replica(self.source_rse_id, self.scope, name1, 1, self.account, session=self.db_session)
         volume = 0
         set_rse_transfer_limits(self.dest_rse_id, self.all_activities, volume=volume, max_transfers=1, session=self.db_session)
         requests = [{
@@ -1801,12 +1843,11 @@ class TestRequestCoreRelease(unittest.TestCase):
         add_replica(self.source_rse_id, self.scope, name1, 1, self.account)
         name2 = generate_uuid()
         add_replica(self.source_rse_id, self.scope, name2, 1, self.account)
-        current_hour = datetime.now().hour
         requests = [{
             'source_rse_id': self.source_rse_id,
             'dest_rse_id': self.dest_rse_id,
             'request_type': RequestType.TRANSFER,
-            'requested_at': datetime.now().replace(hour=current_hour - 2),
+            'requested_at': datetime.now() - timedelta(hours=2),
             'request_id': generate_uuid(),
             'name': name1,
             'scope': self.scope,
