@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2014-2020 CERN
+# Copyright 2014-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,21 +21,18 @@
 # - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2018
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
-
-import logging
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
 
 from flask import Flask, Blueprint, request
-from flask.views import MethodView
 
 from rucio.api.lock import get_dataset_locks_by_rse, get_dataset_locks
-from rucio.common.exception import RucioException, RSENotFound
+from rucio.common.exception import RSENotFound
 from rucio.common.utils import render_json
-from rucio.web.rest.flaskapi.v1.common import check_accept_header_wrapper_flask, parse_scope_name, try_stream, request_auth_env, response_headers
-from rucio.web.rest.utils import generate_http_error_flask
+from rucio.web.rest.flaskapi.v1.common import check_accept_header_wrapper_flask, parse_scope_name, try_stream, \
+    request_auth_env, response_headers, generate_http_error_flask, ErrorHandlingMethodView
 
 
-class LockByRSE(MethodView):
+class LockByRSE(ErrorHandlingMethodView):
     """ REST APIs for dataset locks. """
 
     @check_accept_header_wrapper_flask(['application/x-json-stream'])
@@ -48,30 +45,23 @@ class LockByRSE(MethodView):
         :status 200: OK.
         :status 401: Invalid Auth Token.
         :status 406: Not Acceptable.
-        :status 500: Internal Error.
         :returns: Line separated list of dictionaries with lock information.
         """
+        did_type = request.args.get('did_type', default=None)
+        if did_type != 'dataset':
+            return 'Wrong did_type specified', 500
 
-        did_type = request.args.get('did_type', None)
         try:
-            if did_type == 'dataset':
-                def generate(vo):
-                    for lock in get_dataset_locks_by_rse(rse, vo=vo):
-                        yield render_json(**lock) + '\n'
+            def generate(vo):
+                for lock in get_dataset_locks_by_rse(rse, vo=vo):
+                    yield render_json(**lock) + '\n'
 
-                return try_stream(generate(vo=request.environ.get('vo')))
-            else:
-                return 'Wrong did_type specified', 500
+            return try_stream(generate(vo=request.environ.get('vo')))
         except RSENotFound as error:
-            return generate_http_error_flask(404, error.__class__.__name__, error.args[0])
-        except RucioException as error:
-            return generate_http_error_flask(500, error.__class__.__name__, error.args[0])
-        except Exception as error:
-            logging.exception("Internal Error")
-            return str(error), 500
+            return generate_http_error_flask(404, error)
 
 
-class LockByScopeName(MethodView):
+class LockByScopeName(ErrorHandlingMethodView):
     """ REST APIs for dataset locks. """
 
     @check_accept_header_wrapper_flask(['application/x-json-stream'])
@@ -84,27 +74,22 @@ class LockByScopeName(MethodView):
         :status 200: OK.
         :status 401: Invalid Auth Token.
         :status 406: Not Acceptable.
-        :status 500: Internal Error.
         :returns: Line separated list of dictionary with lock information.
         """
-        did_type = request.args.get('did_type', None)
+        did_type = request.args.get('did_type', default=None)
+        if did_type != 'dataset':
+            return 'Wrong did_type specified', 500
+
         try:
             scope, name = parse_scope_name(scope_name, request.environ.get('vo'))
-            if did_type == 'dataset':
-                def generate(vo):
-                    for lock in get_dataset_locks(scope, name, vo=vo):
-                        yield render_json(**lock) + '\n'
 
-                return try_stream(generate(vo=request.environ.get('vo')))
-            else:
-                return 'Wrong did_type specified', 500
+            def generate(vo):
+                for lock in get_dataset_locks(scope, name, vo=vo):
+                    yield render_json(**lock) + '\n'
+
+            return try_stream(generate(vo=request.environ.get('vo')))
         except ValueError as error:
-            return generate_http_error_flask(400, 'ValueError', error.args[0])
-        except RucioException as error:
-            return generate_http_error_flask(500, error.__class__.__name__, error.args[0])
-        except Exception as error:
-            logging.exception("Internal Error")
-            return str(error), 500
+            return generate_http_error_flask(400, error)
 
 
 def blueprint():
