@@ -23,6 +23,7 @@
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
 # - Eric Vaandering <ewv@fnal.gov>, 2021
+# - Radu Carpa <radu.carpa@cern.ch>, 2021
 
 from datetime import datetime
 from json import dumps, loads
@@ -134,10 +135,8 @@ class Replicas(ErrorHandlingMethodView):
                         yield f'  <glfn name="/atlas/rucio/{rfile["scope"]}:{rfile["name"]}">'
                         yield '</glfn>\n'
 
-                        idx = 0
-                        for replica in replicas:
-                            yield '   <url location="' + str(dictreplica[replica]) + '" priority="' + str(idx + 1) + '">' + escape(replica) + '</url>\n'
-                            idx += 1
+                        for idx, replica in enumerate(replicas, start=1):
+                            yield '   <url location="' + str(dictreplica[replica]) + '" priority="' + str(idx) + '">' + escape(replica) + '</url>\n'
                             if limit and limit == idx:
                                 break
                         yield ' </file>\n'
@@ -303,7 +302,7 @@ class ListReplicas(ErrorHandlingMethodView):
         schemes = param_get(parameters, 'schemes', default=None)
         select = param_get(parameters, 'sort', default=None)
         unavailable = param_get(parameters, 'unavailable', default=False)
-        ignore_availability = 'unavailable' in parameters
+        ignore_availability = param_get(parameters, 'ignore_availability', default='unavailable' in parameters)
         rse_expression = param_get(parameters, 'rse_expression', default=None)
         all_states = param_get(parameters, 'all_states', default=False)
         domain = param_get(parameters, 'domain', default=None)
@@ -322,6 +321,9 @@ class ListReplicas(ErrorHandlingMethodView):
             else:
                 # attempt UTC format '%Y-%m-%dT%H:%M:%S' conversion
                 updated_after = datetime.strptime(updated_after, '%Y-%m-%dT%H:%M:%S')
+        nrandom = param_get(parameters, 'nrandom', default=None)
+        if nrandom:
+            nrandom = int(nrandom)
 
         limit = request.args.get('limit', default=None)
         select = request.args.get('select', default=select)
@@ -349,6 +351,7 @@ class ListReplicas(ErrorHandlingMethodView):
                                            domain=domain, signature_lifetime=signature_lifetime,
                                            resolve_archives=resolve_archives,
                                            resolve_parents=resolve_parents,
+                                           nrandom=nrandom,
                                            updated_after=updated_after,
                                            issuer=issuer,
                                            vo=vo):
@@ -389,10 +392,11 @@ class ListReplicas(ErrorHandlingMethodView):
                         yield f'  <glfn name="/{policy_schema}/rucio/{rfile["scope"]}:{rfile["name"]}"></glfn>\n'
 
                         lanreplicas = [replica for replica, v in dictreplica.items() if v[0] == 'lan']
+                        # sort lan by priority
+                        lanreplicas.sort(key=lambda rep: dictreplica[rep][1])
                         replicas = lanreplicas + sort_replicas({k: v for k, v in dictreplica.items() if v[0] != 'lan'}, client_location, selection=select)
 
-                        idx = 1
-                        for replica in replicas:
+                        for idx, replica in enumerate(replicas, start=1):
                             yield '  <url location="' + str(dictreplica[replica][2]) \
                                 + '" domain="' + str(dictreplica[replica][0]) \
                                 + '" priority="' + str(idx) \
@@ -400,7 +404,6 @@ class ListReplicas(ErrorHandlingMethodView):
                                 + '">' + escape(replica) + '</url>\n'
                             if limit and limit == idx:
                                 break
-                            idx += 1
                         yield ' </file>\n'
 
                 if metalink:
@@ -889,7 +892,7 @@ def blueprint(no_doc=True):
         bp.add_url_rule('', view_func=replicas_view, methods=['post', 'put', 'delete'])
     bp.add_url_rule('/', view_func=replicas_view, methods=['post', 'put', 'delete'])
     suspicious_replicas_view = SuspiciousReplicas.as_view('suspicious_replicas')
-    bp.add_url_rule('/suspicious', view_func=suspicious_replicas_view, methods=['post', ])
+    bp.add_url_rule('/suspicious', view_func=suspicious_replicas_view, methods=['get', 'post'])
     bad_replicas_states_view = BadReplicasStates.as_view('bad_replicas_states')
     bp.add_url_rule('/bad/states', view_func=bad_replicas_states_view, methods=['get', ])
     bad_replicas_summary_view = BadReplicasSummary.as_view('bad_replicas_summary')
@@ -915,7 +918,7 @@ def blueprint(no_doc=True):
 
     if no_doc:
         bp.add_url_rule('/list/', view_func=list_replicas_view, methods=['post', ])
-        bp.add_url_rule('/suspicious/', view_func=suspicious_replicas_view, methods=['post', ])
+        bp.add_url_rule('/suspicious/', view_func=suspicious_replicas_view, methods=['get', 'post'])
         bp.add_url_rule('/bad/states/', view_func=bad_replicas_states_view, methods=['get', ])
         bp.add_url_rule('/bad/summary/', view_func=bad_replicas_summary_view, methods=['get', ])
         bp.add_url_rule('/bad/pfns/', view_func=bad_replicas_pfn_view, methods=['post', ])
