@@ -1,6 +1,6 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
-# Copyright 2013-2020 CERN
+# Copyright 2018-2020 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,13 +15,11 @@
 # limitations under the License.
 #
 # Authors:
-# - Vincent Garonne <vgaronne@gmail.com>, 2013-2017
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2013-2019
-# - Ralph Vigne <ralph.vigne@cern.ch>, 2013
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2014-2019
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2018-2020
-# - Martin Barisits <martin.barisits@cern.ch>, 2018-2019
+# - Martin Barisits <martin.barisits@cern.ch>, 2018-2020
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2018-2020
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
+# - Cedric Serfon <cedric.serfon@cern.ch>, 2018-2019
 # - James Perry <j.perry@epcc.ed.ac.uk>, 2019-2020
 # - Ilija Vukotic <ivukotic@cern.ch>, 2020
 # - Luc Goossens <luc.goossens@cern.ch>, 2020
@@ -35,10 +33,6 @@ from json import dumps, loads
 from traceback import format_exc
 from xml.sax.saxutils import escape
 
-from geoip2.errors import AddressNotFoundError
-from six import string_types
-from web import application, ctx, Created, data, header, InternalError, loadhook, OK, unloadhook
-
 from rucio.api.replica import (add_replicas, list_replicas, list_dataset_replicas, list_dataset_replicas_bulk,
                                delete_replicas, list_dataset_replicas_vp,
                                get_did_from_pfns, update_replicas_states,
@@ -53,12 +47,14 @@ from rucio.common.exception import (AccessDenied, DataIdentifierAlreadyExists, I
                                     ResourceTemporaryUnavailable, RucioException,
                                     RSENotFound, UnsupportedOperation, ReplicaNotFound,
                                     InvalidObject, ScopeNotFound)
-from rucio.common.replica_sorter import sort_random, sort_geoip, sort_closeness, sort_dynamic, sort_ranking
-from rucio.common.schema import get_schema_value
+from rucio.common.schema import insert_scope_name
 from rucio.common.utils import parse_response, APIEncoder, render_json_list
-from rucio.db.sqla.constants import BadFilesStatus, ReplicaState
+from rucio.core.replica_sorter import sort_replicas
+from rucio.db.sqla.constants import BadFilesStatus
 from rucio.web.rest.common import rucio_loadhook, rucio_unloadhook, RucioController, check_accept_header_wrapper
 from rucio.web.rest.utils import generate_http_error
+from six import string_types
+from web import application, ctx, Created, data, header, InternalError, loadhook, OK, unloadhook
 
 try:
     from urllib import unquote
@@ -67,21 +63,21 @@ except ImportError:
     from urllib.parse import unquote
     from urllib.parse import parse_qs
 
-URLS = ('/list/?$', 'ListReplicas',
-        '/?$', 'Replicas',
-        '/suspicious/?$', 'SuspiciousReplicas',
-        '/bad/states/?$', 'BadReplicasStates',
-        '/bad/summary/?$', 'BadReplicasSummary',
-        '/bad/dids/?$', 'BadDIDs',
-        '/bad/pfns/?$', 'BadPFNs',
-        '/rse/(.*)/?$', 'ReplicasRSE',
-        '/bad/?$', 'BadReplicas',
-        '/dids/?$', 'ReplicasDIDs',
-        '%s/datasets$' % get_schema_value('SCOPE_NAME_REGEXP'), 'DatasetReplicas',
-        '/datasets_bulk/?$', 'DatasetReplicasBulk',
-        '%s/datasets_vp$' % get_schema_value('SCOPE_NAME_REGEXP'), 'DatasetReplicasVP',
-        '%s/?$' % get_schema_value('SCOPE_NAME_REGEXP'), 'Replicas',
-        '/tombstone/?$', 'Tombstone')
+URLS = insert_scope_name(('/list/?$', 'ListReplicas',
+                          '/?$', 'Replicas',
+                          '/suspicious/?$', 'SuspiciousReplicas',
+                          '/bad/states/?$', 'BadReplicasStates',
+                          '/bad/summary/?$', 'BadReplicasSummary',
+                          '/bad/dids/?$', 'BadDIDs',
+                          '/bad/pfns/?$', 'BadPFNs',
+                          '/rse/(.*)/?$', 'ReplicasRSE',
+                          '/bad/?$', 'BadReplicas',
+                          '/dids/?$', 'ReplicasDIDs',
+                          '%s/datasets$', 'DatasetReplicas',
+                          '/datasets_bulk/?$', 'DatasetReplicasBulk',
+                          '%s/datasets_vp$', 'DatasetReplicasVP',
+                          '%s/?$', 'Replicas',
+                          '/tombstone/?$', 'Tombstone'))
 
 
 class Replicas(RucioController):
@@ -119,6 +115,14 @@ class Replicas(RucioController):
             if 'limit' in params:
                 limit = int(params['limit'][0])
 
+        client_ip = ctx.env.get('HTTP_X_FORWARDED_FOR')
+        if client_ip is None:
+            client_ip = ctx.ip
+
+        client_location = {'ip': client_ip,
+                           'fqdn': None,
+                           'site': None}
+
         # Resolve all reasonable protocols when doing metalink for maximum access possibilities
         if metalink and schemes is None:
             schemes = SUPPORTED_PROTOCOLS
@@ -140,23 +144,15 @@ class Replicas(RucioController):
                     __first = False
 
                 # ... then, stream the replica information
-                client_ip = ctx.env.get('HTTP_X_FORWARDED_FOR')
-                if client_ip is None:
-                    client_ip = ctx.ip
-
                 replicas = []
                 dictreplica = {}
                 for rse in rfile['rses']:
                     for replica in rfile['rses'][rse]:
                         replicas.append(replica)
                         dictreplica[replica] = rse
-                if select == 'geoip':
-                    try:
-                        replicas = sort_geoip(dictreplica, client_ip)
-                    except AddressNotFoundError:
-                        pass
-                else:
-                    replicas = sort_random(dictreplica)
+
+                replicas = sort_replicas(dictreplica, client_location, selection=select)
+
                 if not metalink:
                     yield dumps(rfile) + '\n'
                 else:
@@ -339,8 +335,11 @@ class ListReplicas(RucioController):
         dids, schemes, select, unavailable, limit = [], None, None, False, None
         ignore_availability, rse_expression, all_states, domain = False, None, False, None
         signature_lifetime, resolve_archives, resolve_parents = None, True, False
-        updated_after = None
-        client_location = {}
+        nrandom, updated_after = None, None
+
+        client_location = {'ip': client_ip,
+                           'fqdn': None,
+                           'site': None}
 
         json_data = data()
         try:
@@ -352,13 +351,14 @@ class ListReplicas(RucioController):
             if 'unavailable' in params:
                 unavailable = params['unavailable']
                 ignore_availability = True
+            if 'ignore_availability' in params:
+                ignore_availability = params['ignore_availability']
             if 'all_states' in params:
                 all_states = params['all_states']
             if 'rse_expression' in params:
                 rse_expression = params['rse_expression']
             if 'client_location' in params:
-                client_location = params['client_location']
-                client_location['ip'] = params['client_location'].get('ip', client_ip)
+                client_location.update(params['client_location'])
             if 'sort' in params:
                 select = params['sort']
             if 'domain' in params:
@@ -367,7 +367,8 @@ class ListReplicas(RucioController):
                 resolve_archives = params['resolve_archives']
             if 'resolve_parents' in params:
                 resolve_parents = params['resolve_parents']
-
+            if 'nrandom' in params:
+                nrandom = params['nrandom']
             if 'signature_lifetime' in params:
                 signature_lifetime = params['signature_lifetime']
             else:
@@ -416,6 +417,7 @@ class ListReplicas(RucioController):
                                        domain=domain, signature_lifetime=signature_lifetime,
                                        resolve_archives=resolve_archives,
                                        resolve_parents=resolve_parents,
+                                       nrandom=nrandom,
                                        updated_after=updated_after,
                                        issuer=ctx.env.get('issuer'),
                                        vo=ctx.env.get('vo')):
@@ -461,30 +463,21 @@ class ListReplicas(RucioController):
                                                                         rfile['scope'],
                                                                         rfile['name'])
 
-                    # TODO: deprecate this
-                    if select == 'geoip':
-                        replicas = sort_geoip(dictreplica, client_location['ip'])
-                    elif select == 'closeness':
-                        replicas = sort_closeness(dictreplica, client_location)
-                    elif select == 'dynamic':
-                        replicas = sort_dynamic(dictreplica, client_location)
-                    elif select == 'ranking':
-                        replicas = sort_ranking(dictreplica, client_location)
-                    elif select == 'random':
-                        replicas = sort_random(dictreplica)
-                    else:
-                        replicas = sorted(dictreplica, key=dictreplica.get)
+                    lanreplicas = [replica for replica, v in dictreplica.items() if v[0] == 'lan']
+                    # sort lan by priority
+                    lanreplicas.sort(key=lambda rep: dictreplica[rep][1])
+                    replicas = lanreplicas + sort_replicas({k: v for k, v in dictreplica.items() if v[0] != 'lan'}, client_location, selection=select)
 
-                    idx = 0
+                    idx = 1
                     for replica in replicas:
                         yield '  <url location="' + str(dictreplica[replica][2]) \
                             + '" domain="' + str(dictreplica[replica][0]) \
-                            + '" priority="' + str(dictreplica[replica][1]) \
+                            + '" priority="' + str(idx) \
                             + '" client_extract="' + str(dictreplica[replica][3]).lower() \
                             + '">' + escape(replica) + '</url>\n'
-                        idx += 1
                         if limit and limit == idx:
                             break
+                        idx += 1
                     yield ' </file>\n'
 
             if metalink:
@@ -697,7 +690,7 @@ class BadReplicasStates(RucioController):
             if 'state' in params:
                 state = params['state'][0]
             if isinstance(state, string_types):
-                state = BadFilesStatus.from_string(state)
+                state = BadFilesStatus(state)
             if 'rse' in params:
                 rse = params['rse'][0]
             if 'younger_than' in params:
@@ -944,7 +937,7 @@ class BadDIDs(RucioController):
                 rse = params['rse']
             if 'reason' in params:
                 reason = params['reason']
-            state = ReplicaState.BAD
+            state = BadFilesStatus.BAD
             if 'expires_at' in params and params['expires_at']:
                 expires_at = datetime.strptime(params['expires_at'], "%Y-%m-%dT%H:%M:%S.%f")
             not_declared_files = add_bad_dids(dids=dids, rse=rse, issuer=ctx.env.get('issuer'), state=state,
